# E.S.O. - VLT project/ESO Archive
# @(#) $Id: vmAstroCat.tcl,v 3.24 2004/12/18 04:03:44 vltsccm Exp $
#
# vmAstroCat.tcl - user interface class for viewing catalog info
#
# See man page vmAstroCat(n) for a complete description.
#
# who         when       what
# --------   ---------   ----------------------------------------------
# D.Bottini 01 Apr 00   created
# aaguayo   2003-03-??  Paranalisation 
# aaguayo   2003-03-14  Fixed checking of Heder for Pre-Image file
# aaguayo   2003-09-12  Fixed bug reported in PPRS-00010716 (curved slits 
#			not corrected saved in final adp files)
# aaguayo   2003-09-12  Fixed bug reported in PPRS-00010720 (wrong flags in 
#		 	temporal VIMOS catalog - during Assign Slits.
# jbrancac  2004-03-03  Add checking of the adp file's length warning.
# mkiekebu  2004-05-16  added checksum to the adp file
# mkiekebu  2004-12-17  PPRS-14666
#

itk::usual vmAstroCat {}

# The vmAstroCat widget class defines a top level widget for searching and
# displaying astronomical catalog data. It contains a menubar with items
# for loading, editing, and saving catalog data and a table displaying
# rows and columns of catalog data. This class does not know anything
# about images or plotting objects in images, however these features may
# be added in a derived class (see the SkySearch(n) in the skycat
# package for an example).
#
# You can also run this class as a standalone application with the
# command "astrocat". The application options are the same as the class
# options.

itcl::class cat::vmAstroCat {
    inherit util::TopLevelWidget

    # constructor

    constructor {args} {
	eval itk_initialize $args
    }

    
    # destructor - delete C++ based objects so that the temp 
    # files are deleted

    destructor {
	# delete temp files on close

	catch {
	    if {"[$w_.cat servtype]" == "local"} {
		if {"[string range $itk_option(-catalog) 0 4]" == "/tmp/"} {
		    file delete $itk_option(-catalog)
		}
	    }
	}
	catch {$w_.cat delete}
	catch {close $rfd_}
	catch {close $wfd_}
	catch {unset instances_($instance_idx_)}
	if {"$current_instance_" == "$w_"} {
	    set current_instance_ {}
	}
	#ExitApp
	# AAG - Here it is deleting all adp_cat files
        #catch {eval "file delete [glob -nocomplain [file dirname $itk_option(-catalog)]/*.adp_cat]"}
    }

    
    # called after options have been evaluated
########################## Modified for vmmps #########################
    protected method init {} {
	global adpcat rootbase mps_window slit_window slitcat_window

	#puts "DB> Initialising vmAstroCat"
	#puts "DB> windows: $mps_window, $mps_window, $slitcat_window"
	# if running stand-alone, setup X defaults
	if {$itk_option(-standalone)} {
	    util::setXdefaults
	    cat::setXdefaults  
	}
	# set the file used to keep track of URLs (for debugging)
	set_logfile
	
	# do window layout
	layout_dialog

	# enter widget name in instance array for later reference
	set_instance

	# add a short help window
	make_short_help

	# create an object for running interruptable batch queries
	Batch $w_.batch \
	    -command [code $this preview_done] \
	    -debug $itk_option(-debug)

	# position from previous window, if pos.
	if {"$current_instance_" != ""} {
	    wm geometry $w_ +[winfo x $current_instance_]+[winfo y $current_instance_]
	}
	set current_instance_ $w_

	# for local catalogs, start search automatically
	set name $itk_option(-catalog)
	set rootbase $itk_option(-catalog)
	if {"[$w_.cat servtype]" == "local"} {
	    wm title $w_ "[file tail [$w_.cat longname $name]] ($itk_option(-number))"
	    wm iconname $w_ "[file tail [$w_.cat shortname $name]]"
	    search
	}

	# check for TCS catalog
	if {[$w_.cat is_tcs] || \
		(! $itk_option(-tcs) \
		     && [file exists $name] \
		     && [is_tcs_catalog $name])} {
	    global ::$w_.tcs
	    set $w_.tcs 1
	    set_tcs_columns
	}

	set initialized_ 1

        set cattype ""
        catch {set cattype [exec grep -i X_SLIT $name | grep -i Y_SLIT]}
        if {$cattype == ""} {
           mps_add_dialog_buttons
        } else {
	    if {$cattype != "" && $adpcat == 0} {
		#puts "DB> Loading slit buttons"
		slit_add_dialog_buttons
		slits
	    } else {
		#puts "DB> Loading slitcat buttons"
		slitcat_add_dialog_buttons
	    }
        }

    } 

    
    # create the ~/.skycat dir if it does not already exists and keep a log
    # file there.

    protected method set_logfile {} {
	global ::env

	# open log file used to keep track of URLs (for debugging)
	set dir $env(HOME)/.skycat

	if {! [file isdirectory $dir]} {
	    catch {mkdir $dir}
	}
	set logfile_name_ $dir/log
    }

    
    # open or close a pipe to get feedback during HTTP transfers.
    # (To save limited fds, we close the feedback pipe after each HTTP op).
    # The arg should be "on" to turn feedback on, or "off" to turn it off.

    protected method set_feedback {onoff} {
	# Process any pending file events.
	# Note: this is important: if we don't process the events before
	# closing the feedback file, a crash may result.
	update 

	if {$itk_option(-debug)} {
	    # if -debug was given, the feedback bit wont work, since the
	    # query is done in the foreground and trying to read the feedback
	    # from the pipe may cause the application to hang...
	    return
	}
	if {"$onoff" == "on"} {
	    lassign [pipe] rfd_ wfd_
	    fileevent $rfd_ readable [code $this feedback]
	    $w_.cat feedback $wfd_
	} elseif {[info exists rfd_]} {
	    ::close $rfd_
	    ::close $wfd_
	    unset rfd_ wfd_
	    $w_.cat feedback {}
	}
    }

    
    # keep an array of instances(name,id) to help locate the 
    # window for a catalog
    
    protected method set_instance {} {
	set name [$w_.cat longname $itk_option(-catalog)]
	set id $itk_option(-id)
	set instance_idx_ "$name,$id"
	set instances_($instance_idx_) $w_ 
    }


    # add the menu bar

    protected method add_menubar {} {
	TopLevelWidget::add_menubar

	set m [add_menubutton File "Display File menu"]
	set file_menu_ $m
	
	if {$iscat_} {
	    add_menuitem $m command "Open" \
		{Open a local catalog in tab table format} \
		-command [code cat::vmAstroCat::local_catalog \
			      $itk_option(-id) \
			      [info class] \
			      $itk_option(-debug) $w_] \
		-accelerator "Control-o"
	    
	    $m add separator

	    add_menuitem $m command "Save as..." \
		{Save listed objects to a local catalog file} \
		-command [code $this save_as] \
		-accelerator "Control-s"

	    add_menuitem $m command "Add to..." \
		{Add listed objects to a local catalog file} \
		-command [code $this add_to] \
		-accelerator "Control-a"

	    add_menuitem $m command "Add selected..." \
		{Add selected rows to a local catalog file} \
		-command [code $this add_selected] \
		-accelerator "Control-A"

	    $m add separator

	    add_menuitem $m command "Print..." \
		{Print the listing to a printer or file} \
		-command [code $this print] \
		-accelerator "Control-p"

	    add_menuitem $m command "Clear" \
		{Clear the catalog listing} \
		-command [code $this clear]

	    $m add separator
	}

	add_menuitem $m command "Close" \
	    {Close this window} \
	    -command [code $this close]
	
	if {$itk_option(-standalone)} {
	    add_menuitem $m command "Exit" \
		{Exit the application} \
		-command [code $this quit] \
		-accelerator "Control-q"
	}
	
	if {$iscat_} {
	    # Edit menu
	    set m [add_menubutton Edit "Display Edit menu"]
	    set edit_menu_ $m

	    add_menuitem $m command "Remove selected" \
		{Remove selected rows from the local catalog} \
		-command [code $this remove_selected] \
		-state disabled

	    add_menuitem $m command "Enter new object..." \
		{Enter the data for a new object for the local catalog} \
		-command [code $this enter_new_object] \
		-state disabled

	    add_menuitem $m command "Edit selected object..." \
		{Edit the data for the selected object in the local catalog} \
		-command [code $this edit_selected_object] \
		-state disabled
	}

	# Options menu
	set m [add_menubutton Options "Display Options menu"]
	set options_menu_ $m
	add_menuitem $m cascade "Set Name Server" \
	    {Select the name server used to resolve astronomical object names} \
	    -menu [set ns_menu_ [menu $itk_component(options).m.ns]]

	if {$iscat_} {
	    add_menuitem $m checkbutton "Use TCS Columns" \
		{Display the standard columns used by the ESO TCS (Telescope Control Software)} \
		-command [code $this set_tcs_columns] \
		-variable $w_.tcs \
		-onvalue 1 \
		-offvalue 0

	    $m add separator

	    add_menuitem $m command "Set Sort Columns..." \
		{Set options for sorting the query results} \
		-command [code $this sort_dialog] \
		-state disabled

	    add_menuitem $m command "Hide/Show Columns..." \
		{Set options for displaying columns of the query results} \
		-command [code $this select_columns] \
		-state disabled
	

	    add_menuitem $m command "Set Search Columns..." \
		{Set the list of columns to search by for this catalog} \
		-command [code $this set_search_cols] \
		-state disabled

	    add_menuitem $m command "Set Plot Symbols..." \
		{Set the symbol (color, size, etc.) to use to plot objects} \
		-command [code $this set_plot_symbols] \
		-state disabled

	    $m add separator
	}
	add_menuitem $m command "Proxies..."  \
	    "Define an HTTP proxy server for use with a firewall." \
	    -command [code cat::vmAstroCat::proxies]
	check_proxies

    }
        

    # enable or disable some menus
    
    protected method set_menu_states {} {
	$options_menu_ entryconfig "Set Sort Columns..." -state normal
	$options_menu_ entryconfig "Hide/Show Columns..." -state normal

	if {[$w_.cat iswcs] || [$w_.cat ispix]} {
	    set state normal
	} else {
	    set state disabled
	}
	$options_menu_ entryconfig "Set Plot Symbols..." -state $state
	$options_menu_ entryconfig "Set Name Server" -state $state
	$options_menu_ entryconfig "Use TCS Columns" -state $state

	# determine states for some menu items
	if {"[$w_.cat servtype]" == "local"} {
	    set state [set sstate normal]
	} else {
	    set state [set sstate disabled]
	    # allow user to edit search cols if URL contains "%cond"
	    if {[string first "%cond" [$w_.cat url]] >= 0} {
		set sstate normal
	    } 
	}
	$edit_menu_ entryconfig "Remove selected" -state $state
	$edit_menu_ entryconfig "Enter new object..." -state $state
	$edit_menu_ entryconfig "Edit selected object..." -state $state

	$options_menu_ entryconfig "Set Search Columns..." -state $sstate
    }
    
    
    # reload the default catalog config file using the given astrocat object

    public proc reload_config_file {w} {
	global ::env
	set config_file $env(HOME)/.skycat/skycat.cfg
	if {[file exists $config_file]} {
	    set env(CATLIB_CONFIG) "file:$config_file"
	}

	if {[catch {$astrocat_ reload} msg]} {
	    error_dialog $msg $w
	}
	# make sure these windows are updated
	catch {destroy $w_.symconf}
	catch {destroy $w_.searchconf}
	if {[winfo exists .catinf]} {
	    .catinf reinit_tree
	}
	
	if {[info exists instances_]} {
	    foreach i [array names instances_] {
		$instances_($i) update_search_options
	    }
	}
	update_catalog_menus
    }


    # Check for a file ~/.skycat/proxies, once each session, and
    # use it to initialize environment variables for a proxy server
    # (see also tclutil/util/src/HTTP.C).

    public proc check_proxies {} {
	global ::env
	# only do it once
	if {$checked_proxies_} {
	    return 
	}
	cat::ProxyDialog::check_proxies $env(HOME)/.skycat/proxies
	set checked_proxies_ 1
    }

    # Pop up a dialog to set or change the HTTP proxy server.

    public proc proxies {} {
	global ::env
	utilReUseWidget ProxyDialog .proxy \
	    -configfile $env(HOME)/.skycat/proxies
    }


    # pop up a window to browse the catalog directories. 
    #
    # id is an optional unique id to be associated with a new catalog widget.
    #
    # classname is the name of the vmAstroCat subclass to use to create new
    # catalog widgets (defaults to "vmAstroCat").
    #
    # debug is a flag is passed from the command line. If true, querries
    # are run in the foreground, to make debugging easier.
    #
    # w should be the caller's top level window, if specified.
    #

    proc catalog_directory {{id ""} {classname vmAstroCat} {debug 0} {w ""}} {
	utilReUseWidget cat::CatalogInfo .catinf \
	    -id $id \
	    -classname $classname \
	    -debug $debug \
	    -callerw $w \
	    -command [code cat::vmAstroCat::update_catalog_menus]
	
	# put the window clone number in the title
	if {[winfo exists $w]} {
	    catch {wm title .catinf "Catalog Directory ([$w cget -number])"}
	}
    }

    
    # update all of the catalog menus in all instances to show 
    # the current catalog info. 

    public proc update_catalog_menus {} {

	foreach i [array names catalog_menu_info_] {
	    if {[winfo exists [utilNamespaceTail $i]]} {
		eval $catalog_menu_info_($i)
	    }
	}
    }
    

    # pop up a dialog to set the plot symbols to use for this catalog

    public method set_plot_symbols {} {
	set columns $headings_
	if {[llength $columns] == 0} {
	    info_dialog "Please make a query first so that the column names are known" $w_
	    return
	}
	utilReUseWidget cat::SymbolConfig $w_.symconf \
	    -catalog $itk_option(-catalog) \
	    -astrocat [code $w_.cat] \
	    -columns $columns \
	    -command [code $this plot]
    }


    # pop up a dialog to set the search columns to use for this catalog

   public  method set_search_cols {} {
	set columns $headings_
	if {[llength $columns] == 0} {
	    info_dialog "Please make a query first so that the column names are known" $w_
	    return
	}
	utilReUseWidget cat::SearchConfig $w_.searchconf \
	    -catalog $itk_option(-catalog) \
	    -command [code $this update_search_options] \
	    -astrocat [code $w_.cat] \
	    -columns $columns
    }


    # update the search option entries after they have been edited

    public method update_search_options {} {
	$searchopts_ update_search_options
    }

    # close this window

    public method close {} {
	if {$itk_option(-standalone)} {
	    wm iconify  $w_
	} else {
	    wm withdraw $w_
	}
 #       catch {eval "file delete [glob -nocomplain [file dirname $itk_option(-catalog)]/*.adp_cat]"}
    }

    
    # pop up a dialog to sort the list

    public method sort_dialog {} {
	$results_ sort_dialog
    }

    
    # called when the user has selected columns to sort the results by.
    # The first arg is the sort columns, the second arg is the order
    # (increasing, decreasing)
    
    public method set_sort_cols {sort_cols sort_order} {
	global ::$w_.tcs
	if {"[$w_.cat sortcols]" != "$sort_cols" \
		|| "[$w_.cat sortorder]" != "$sort_order"} {
	    $w_.cat sortcols $sort_cols
	    $w_.cat sortorder $sort_order
	    $w_.cat is_tcs $itk_option(-catalog) [set $w_.tcs]
	    cat::CatalogInfo::save {} $w_ 0
	    $results_ config -sort_cols $sort_cols -sort_order $sort_order
	    search
	}
    }


    # pop up a dialog to select table columns to display

    public method select_columns {} {
	$results_ select_columns
    }


    # called when the user has selected columns to show
    
    public method set_show_cols {cols} {
	global ::$w_.tcs
	set show [$w_.cat showcols]
	if {"$show" == ""} {
	    set show [$results_ get_headings]
	}
	if {"$show" != "$cols"} {
	    $w_.cat showcols $cols
	    $w_.cat is_tcs $itk_option(-catalog) [set $w_.tcs]
	    cat::CatalogInfo::save {} $w_ 0
	}
    }


    # add the name server catalogs to the name server menu

    protected method get_name_servers {} {
	set m $ns_menu_
	if {[catch {set list [$w_.cat info namesvr]} msg]} {
	    error_dialog $msg $w_
	    return
	}

	foreach namesvr $list {
	    $m add radiobutton \
		-label $namesvr \
		-command [code $this set_namesvr $namesvr] \
		-variable $m
	}

	if {![info exists namesvr]} {
	    error_dialog "No default name server found for astronomical objects" $w_
	    return
	}

	# set default name server
	global ::$m
	set $m $namesvr
	set_namesvr $namesvr
    }


    # toggle the TCS option (use the tcscat or the astrocat Tcl command)

    public method set_tcs_columns {} {
	global ::$w_.tcs
	set is_tcs [set $w_.tcs]
	set itk_option(-tcs) $is_tcs
	catch {$w_.cat delete}

	if {$is_tcs} {
	    tcscat $w_.cat
	} else {
	    astrocat $w_.cat
	}
	if {[catch {$w_.cat open $itk_option(-catalog)} msg]} {
	    error_dialog $msg $w_
	    return
	}
	
	set iscat_ 1

	# if changing formats, reset table column info and entry
	# since column names will change making previous info invalid
	if {[$w_.cat is_tcs] != $is_tcs} {
	    $w_.cat is_tcs $itk_option(-catalog) $is_tcs
	    set reset_columns_ 1
	}

	if {[llength $headings_]} {
	    search
	}
    }


    # reset table dialogs if needed

    public method reset_table {} {
	$results_ reset
	$results_ set_options {MORE PREVIEW more preview} Show 0
	$w_.cat showcols {}
	$w_.cat sortcols {}
	cat::CatalogInfo::save {} $w_ 0
    }

    
    # return true if the given file contains a TCS catalog

    public proc is_tcs_catalog {filename} {
	if {[catch {set f [open $filename]}]} {
	    return 0
	}
	set title [gets $f]
	::close $f
	if {[string match "Tcs*" $title]} {
	    return 1
	}
	return 0
    }

    
    # clear the table listing
    
    public method clear {} {
	catch {$results_ clear}
    }

    
    # pop up a dialog to print the table listing
    
    public method print {} {
	$results_ print_dialog
    }

       
    # Ask the user for the name of a local catalog file and then
    # open a window for the catalog.
    #
    # id is an optional unique id to be associated with a new catalog widget.
    #
    # classname is the name of the vmAstroCat subclass to use to create new
    # catalog widgets (defaults to "vmAstroCat").
    #
    # $debug is a flag set from the command line arg.
    #
    # $w should be the top level window of the caller, if specified.
    
    public proc local_catalog {{id ""} {classname vmAstroCat} {debug 0} {w ""} {ext cat} {type ""}} {

      global imaname
      global flag_use

      set flag_use 0
      set contributed 0

      if {$ext == "fits"} {
	  error_dialog "FITS catalog not supported since version 3.2" .skycat1
      } else {
	  set catname_in [filename_dialog [pwd] *.$ext .skycat1]
          if {"$catname_in" != ""} {
	      if {[file isfile $catname_in]} {
		  if {$ext != "adp"} {
		      # AAG- Check if it is a contributed or Vimos Catalog
		      set number_vimos ""
		      catch {set number_vimos [exec grep -i X_IMAGE $catname_in]}
		      if {$number_vimos == "" && $type!="spoc"} {
			  set contributed 1
			  #puts "DB> Contributed catalog - input: $catname_in"
			  set catname [ascii2skycat $catname_in]
		      } else {
			  set catname [ascii2vimos $catname_in]
			  if {$catname == 0} {
			    return
			  }
			  #puts "DB> VIMOS catalog - input: $catname_in"
		      }
		  } else {
			  set catname $catname_in
			  #puts "DB> ADP File - input: $catname_in"
		  }
		  if {[catch {$astrocat_ check $catname} msg]} {
		      error_dialog $msg .skycat1
		  } else {
		      if {$ext == "adp"} {
			  conv_from_adp $catname
			  set catname $catname\_cat		       
		      }
		      set vimos ""
		      catch {set vimos [exec grep VIMOS $catname]}
		      set instrume ""
		      set instrume [image2 fits get "INSTRUME"]
		      set imaname [image2 cget -file]
		      ### AAG - should be? ... if $vimos == "" && $instrume != "VIMOS"
		      if {$instrume != "VIMOS"} {
			  info_dialog "Select the VIMOS pre-image" .skycat1
			  puts "Opening file ..."
			  #.skycat1.image open			
			  .skycat1.image rtd::RtdImageCtrl::open
			  update 
			  update idletasks			 
		      }
		      if {$vimos!="" && $ext != "adp"} {
			 warning_dialog "Please Check that the loaded catalog is in the correct format!" $w
		      }
		      	select_catalog $catname local $id $classname [is_tcs_catalog $catname] $debug $w
		  }
	      } else {
		  error_dialog "There is no file named '$catname'" .skycat1
	      }
	  }
      }
  }

    public proc ascii2skycat { catname } {

	#puts "DB> Converting $catname to skycat format"
	set line ""
	set ascii_file $catname
	set skycat_file [file rootname $catname].cat_tmp
#	file rename -force $catname $ascii_file
	
	if {[file exists $ascii_file] && [file readable $ascii_file]} {
	    #puts "DB> Opening $ascii_file for reading"
	    set fId [::open $ascii_file r ]
	    # as long as there are lines in the file
	    set Nrow 0 
	    set indexID -1
	    set indexRA -1
	    set indexDEC -1
	    set val_cat ""
	    set flag_header 0
	    
	    while {[gets $fId line] >= 0} {
	        set line [string toupper $line]
		if {[string match #* $line] || [string match #--- $line]} {
		     # ignore comment lines
		     # puts stdout $line
		 } else {
		     if {$flag_header == 0} {
 		         set l [llength $line]
		         if {$l == 0} {
			   error_dialog "Please remove empty rows in catalog" .skycat1
			   return 0
		         }

		         # ---------------------------
		         # PART-1 Header
		         # ---------------------------
		         if {[string match *X_IMAGE* $line]} {
			    error_dialog "Warning: X_IMAGE is not allowed in contributed catalog" .skycat1
			    return 0
		         }
		         if {[string match *Y_IMAGE* $line]} {
			    error_dialog "Warning: Y_IMAGE is not allowed in contributed catalog" .skycat1
			    return 0
		         }

			 set indexID [lsearch -exact $line ID]
			 if {$indexID == -1} {
			     error_dialog "Warning:$ascii_file does not have the mandatory column ID" .skycat1
			     return 0
			 }
			 set line [lreplace $line $indexID $indexID]
			 set indexRA [lsearch -exact $line RA]
			 if {$indexRA == -1} {
			     error_dialog "Warning:$ascii_file does not have the mandatory column RA" .skycat1
			     return 0
			 }
			 set line [lreplace $line $indexRA $indexRA]
			 set indexDEC [lsearch -exact $line DEC]
			 if {$indexDEC == -1} {
			     error_dialog "Warning:$ascii_file does not have the mandatory column DEC" .skycat1
			     return 0
			 }			 

			 set flag_header 1
			 set header1 "ID RA DEC"

			 set header2 [lreplace $line $indexDEC $indexDEC]
			 set header2 [string toupper $header2]
			 set header [concat $header1 $header2]
			 set res [regsub -all " " $header \t header]
 		         set cols [llength $header]
		     } else {
			 set l [llength $line] 
			 if {$l != $cols} {
			     error_dialog "Row [expr $Nrow + 2 ]: Found an empty value. Please check" .skycat1
			     return 0
			 }
			 if {$l > 0 && $indexID != -1 && $indexRA != -1 && $indexDEC != -1} {
			     set val_id [lindex $line $indexID]
			     if {[regexp {^[0-9]+$} $val_id result] != 1 } {
			         error_dialog "Row [expr $Nrow + 2]: ID=$val_id is an invalid integer value" .skycat1
			         return 0
			     }

			     set line [lreplace $line $indexID $indexID]
			     set val_ra [lindex $line $indexRA]
			     set line [lreplace $line $indexRA $indexRA]
			     set val_dec [lindex $line $indexDEC]
			     set val_other [lreplace $line $indexDEC $indexDEC]
			     set val_line [concat $val_id $val_ra $val_dec $val_other]
			     set res [regsub -all " " $val_line \t val_line]
			     set val_cat $val_cat$val_line\n

			     set Nrow [expr $Nrow + 1]
			 }
		     }
		 }
		}
	        ::close $fId
	    } else {
		error_dialog "Warning:$acii_file: does not exist or has no read permission" .skycat1
		return 0
	    }
	    
	    #puts "DB> Opening $skycat_file for writing"
            set fileId [::open $skycat_file w ]
            puts $fileId "QueryResult"
            puts $fileId ""
            puts $fileId "# Config entry"
            puts $fileId "serv_type: local"
            puts $fileId "short_name: $skycat_file"
            puts $fileId "long_name_name: $skycat_file"
            puts $fileId "url: $skycat_file"
            puts $fileId "# End config entry"
            puts $fileId ""

            puts $fileId "$header"
            puts $fileId "---"
            puts -nonewline $fileId $val_cat
            ::close $fileId

	    # AAG - If cross column is not present, we add a CROSS column 
	    # with calue=1 fro everything

	    set heading [exec grep ID $skycat_file | grep RA | grep DEC]
	    set indexCROSS [lsearch -exact $heading CROSS]
	    if {$indexCROSS == -1} {
		#puts "DB> CROSS column was not found in $skycat_file. Adding..."
	        set _cross [open $skycat_file\_cross w]
		for {set n 1} {$n <= 9} {incr n 1} {
		    puts $_cross ""
	 	}
		puts $_cross CROSS
		puts $_cross ""
		for {set n 1} {$n <= $Nrow} {incr n 1} {
		    puts $_cross 1
		}
		::close $_cross

		exec paste $skycat_file $skycat_file\_cross > $skycat_file\_all
		catch {file rename -force $skycat_file\_all $skycat_file}
		catch {file delete $skycat_file\_cross}
	    }
            return $skycat_file

	}
    

    public proc ascii2txt { file } {

	set line ""
	set ascii_file $file
	set txt_file [file rootname $file].txt_tmp
#	file rename -force $file $ascii_file
	
	if {[file exists $ascii_file] && [file readable $ascii_file]} {
	    #puts "DB> Opening $ascii_file for reading"
	    set fId [::open $ascii_file r ]
	    # as long as there are lines in the file
	    set Ncol -1
	    set indexID -1
	    set indexX -1
	    set indexY -1
	    set val_cat ""
	    
	    while {[gets $fId line] >= 0} {
	        set line [string toupper $line]
		#puts "Line in $ascii_file: $line"
		if {[string match #* $line] || [string match #--- $line]} {
		     # ignore comment lines
		     # puts stdout $line
		 } else {
		     # ---------------------------
		     # PART-1 Header
		     # ---------------------------
		     set l [llength $line]
	  	     if {$l == 0} {
		        error_dialog "Please remove empty row in catalog" .skycat1
			return 0
		     }
		     if {[string match *ID* $line]} {
			 set Ncol [llength $line]
			 set indexID [lsearch -exact $line ID]
			 if {$indexID == -1} {
			     error_dialog "Warning:$ascii_file does not have the mandatory column ID" .skycat1
			     return -1
			 }
			 set indexX [lsearch -exact $line X_IMAGE]
			 if {$indexX == -1} {
			     error_dialog "Warning:$ascii_file does not have the mandatory column X_IMAGE" .skycat1
			     return -1
			 }
			 set indexY [lsearch -exact $line Y_IMAGE]
			 if {$indexY == -1} {
			     error_dialog "Warning:$ascii_file does not have the mandatory column Y_IMAGE" .skycat1
			     return -1
			 }			 
		     } else {
			 if {$indexID != -1 && $indexX != -1 && $indexY != -1} {
			     set val_id [lindex $line $indexID]
			     set val_x [lindex $line $indexX]
			     set val_y [lindex $line $indexY]

			     # For the vimoscat in cross only use 2 columns
			     set val_line [concat $val_x $val_y]
			     set val_cat $val_cat$val_line\n
			 }
		     }
		 }
		}
	        ::close $fId
	    } else {
		error_dialog "Warning:$ascii_file: does not exist or has no read permission" .skycat1
		return -1
	    }
	    
	    #puts "DB> Opening $txt_file for writing"
	    set fileId [::open $txt_file w ]
	    puts -nonewline $fileId $val_cat
	    ::close $fileId
	    
	    return $txt_file
	}
    
#   ascii2vimos is just checking the content of the catalog
    public proc ascii2vimos { catname } {

	set line ""
	set ascii_file $catname
#	set vimos_file [file rootname $catname].cat_tmp_
	
	if {[file exists $ascii_file] && [file readable $ascii_file]} {
	    set fId [::open $ascii_file r ]
	    # as long as there are lines in the file
	    set Nrow 0 
	    set indexID -1
	    set indexX -1
	    set indexY -1
	    set val_cat ""
	    set flag_header 0
	    
	    while {[gets $fId line] >= 0} {
	        set line [string toupper $line]
		if {[string match #* $line] || [string match #--- $line]} {
		     # ignore comment lines
		     # puts stdout $line
		 } else {
		     if {$flag_header == 0} {
		         # ---------------------------
		         # PART-1 Header
		         # ---------------------------
 		     
			 set l [llength $line]
	  	         if {$l == 0} {
			  error_dialog "Please remove empty row in catalog" .skycat1
			   return 0
		         }

			 set indexID [lsearch -exact $line ID]
			 if {$indexID == -1} {
			     error_dialog "Warning:$ascii_file does not have the mandatory column ID" .skycat1
			     return 0
			 }

			 set indexX [lsearch -exact $line X_IMAGE]
			 if {$indexX == -1} {
			     error_dialog "Warning:$ascii_file does not have the mandatory column X_IMAGE" .skycat1
			     return 0
			 }

			 set indexY [lsearch -exact $line Y_IMAGE]
			 if {$indexY == -1} {
			     error_dialog "Warning:$ascii_file does not have the mandatory column Y_IMAGE" .skycat1
			     return 0
			 }			 

			 set flag_header 1

			 set header [string toupper $line]
 		         set cols [llength $header]
		     } else {
			 set l [llength $line] 
			 if {$l != $cols} {
			     error_dialog "Row [expr $Nrow + 2]: Found an empty value. Please check" .skycat1
			     return 0
			 }
			 if {$l > 0 && $indexID != -1 && $indexX != -1 && $indexY != -1} {
			     set val_id [lindex $line $indexID]
			     if {[regexp {^[0-9]+$} $val_id result] != 1 } {
			         error_dialog "Row [expr $Nrow + 2]: ID=$val_id is an invalid integer value" .skycat1
			         return 0
			     }

			     set val_cat $val_cat$line\n

			     set Nrow [expr $Nrow + 1]
			 }
		     }
		 }
		}
	        ::close $fId
	    } else {
		error_dialog "Warning:$acii_file: does not exist or has no read permission" .skycat1
		return 0
	    }
	    
    #        set fileId [::open $vimos_file w ]
    #        puts $fileId "$header"
    #        puts -nonewline $fileId $val_cat
    #        ::close $fileId
    #        return $vimos_file

	    return $ascii_file

	}


    # Fill up a menu of known data servers to choose from.
    # 
    # w is the caller's top level widget (containing menubar)
    #
    # id is an optional unique id to be associated with a new catalog widget.
    #
    # classname is the name of the vmAstroCat subclass to use to create new
    # catalog widgets (defaults to "vmAstroCat").
    #
    # $debug is a flag set from the command line arg.
    #
    # The serv_type argument should be one of: "catalog", "archive".
    #
    # An optional "cmd" argument may be set to longname or shortname
    # depending on the names you want in the menu (for local catalogs,
    # specify shortname, to avoid getting the complete path name in
    # the menu.
    
    public proc fill_catalog_menu {w id classname m debug serv_type {cmd longname}} {
	if {[catch {set catalog_list [lsort [$astrocat_ info $serv_type]]} msg]} {
	    error_dialog $msg $w
	    return
	}
	
	if {[llength $catalog_list]} {
	    foreach i $catalog_list {
		set name [$astrocat_ $cmd $i]
		set is_tcs [$astrocat_ is_tcs $i]
		$w add_menuitem $m command $name \
		    "Open $serv_type: \"$i\"" \
		    -command [code cat::vmAstroCat::select_catalog $i $serv_type $id \
				  $classname $is_tcs $debug $w]
	    }
	}
    }

    
    # Open a window for the given catalog, or report an error if the catalog
    # does not exist. 
    # $name is the name of the catalog to open,
    # $classname is the class to use to open it,
    # $debug is an optional debugging flag,
    # $w should be set to the top level window of the caller (optional).


    public proc open_catalog_window {name {id ""} {classname vmAstroCat} {debug 0} {w ""}} {

	if {[catch {$astrocat_ open $name} msg]} {
	    error_dialog $msg $w
	    return
	}

	cat::vmAstroCat::select_catalog $name catalog $id $classname 0 $debug $w
    }


    # save the current data to a local catalog
    
    public method save_as {} {
	$results_ save_as
    }


    # add the rows in the current listing to a local catalog file
    
    public method add_to {} {
	$results_ add_to
    }

    
    # add the currently selected rows to a local catalog file

    public method add_selected {} {
	$results_ add_selected
    }

    
    # remove the currently selected rows from a local catalog file

    public method remove_selected {} {
	$results_ remove_selected

	# update the display
	clear
	clearsymbol
	#clearslits
	slits
	search
    }

    
    # pop up a dialog to enter the data for a new object for a local catalog

    public method enter_new_object {} {
	$results_ enter_new_object [code $this search]
    }

    
    # pop up a window so that the user can edit the selected object(s)

    public method edit_selected_object {} {
	$results_ edit_selected_object [code $this search]
    }

#    public method create_slits {} {
#    	set target_canvas_ .skycat1.image.imagef.canvas
#    	$target_canvas_ addtag slit withtag xx 
#    	$target_canvas_ bind $slit <Button-1> [code $this create_slit %W %x %y 0 ]
#    }

#    public method create_slit { w sx sy but {on 1} {click 1} } {

#     # creates a new slit for the FORS2-MXU
#     # sx,sy screen coords, but = mouse button
#     # stop==1: do nothing (when selecting a reference object)
#     # stop==0: do the normal procedure

#     puts "DB> create_slit : $w sx=$sx sy=$sy but=$but"
#     if {$stop} { return }
#    }


########################## New method for vmmps #########################
    public method slits {} {
        global catname
        set bg black
        set fg white
        set tags mark
        set target_canvas_ .skycat1.image.imagef.canvas
        set target_image_ image2

	clearslits
	#puts "JLBQ >> ............. $catname ..............................."
	#puts "JLBQ >> [expr [exec grep Spectrum $catname | cut -f2- -d:] / 2.0] ..... $catname"
        set spect_len [expr [exec grep Spectrum $catname | cut -f2- -d:] / 2.0]
	#puts "JLBQ >> ............................................"

        $target_image_ convert dist $spect_len $spect_len image spect_len spect_len canvas

        set nobj [$results_ total_rows]
        for {set n 0} {$n < $nobj} {incr n} {
          set row [lindex $info_ $n]
          set x [lindex $row 3]
          set y [lindex $row 4]
          set dimx [expr [lindex $row 5] / 2.0]
          set dimy [expr [lindex $row 6] / 2.0]
        
          $target_image_ convert coords $x $y image x y canvas
          $target_image_ convert dist $dimx $dimy image dimx dimy canvas

          set x1 [expr $x - $dimx]
          set y1 [expr $y + $dimy]
          set x2 [expr $x + $dimx]
          set y2 [expr $y + $dimy]
          set x3 [expr $x + $dimx]
          set y3 [expr $y - $dimy]
          set x4 [expr $x - $dimx]
          set y4 [expr $y - $dimy]
          set yy1 [expr $y + $spect_len]
          set yy2 [expr $y + $spect_len]
          set yy3 [expr $y - $spect_len]
          set yy4 [expr $y - $spect_len]
                  
          $target_canvas_ create line $x1 $yy1 $x2 $yy2 \
            -fill $fg \
            -width 1 \
            -tags "$tags $fg"            
          $target_canvas_ create line $x2 $yy2 $x3 $yy3 \
            -fill $fg \
            -width 1 \
            -tags "$tags $fg"            
          $target_canvas_ create line $x3 $yy3 $x4 $yy4 \
            -fill $fg \
            -width 1 \
            -tags "$tags $fg"            
          $target_canvas_ create line $x4 $yy4 $x1 $yy1 \
            -fill $fg \
            -width 1 \
            -tags "$tags $fg"            
          $target_canvas_ create line $x1 $y1 $x2 $y2 \
            -fill $fg \
            -width 1 \
            -tags "$tags $fg"            
          $target_canvas_ create line $x3 $y3 $x4 $y4 \
            -fill $fg \
            -width 1 \
            -tags "$tags $fg"            
          

        }            
        return
    }

    public method clearsymbol {} {
	::skycat::vmSkySearch::delete_objects
	return
    }	

    public method clearslits {} {
        set target_canvas_ .skycat1.image.imagef.canvas
        $target_canvas_ delete mark

        set draw [.skycat1.image component draw]
	$draw deselect_objects

        return
    }


######################## New method for vmmps ########################
    public method _sel {flag {del 0}} {
      set row [lindex [$results_ get_selected] 0]
      if {$row == ""} {
        error_dialog "You must select a plotted object before click this button" .skycat1
        return
      }
      if {[lindex $row end] == "A" && $del == 0} {
        error_dialog "You can't delete a Curved Slit with this button, use Curved Slit button and its Del sub-button" .skycat1
        return
      }
      set newrow [lreplace $row end end $flag]
      $results_ enter_object "namespace inscope ::cat::vmAstroCat {$this search}" $newrow
    }

    # open the catalog for this window

    public method open_catalog {} {


	# create astrocat object (or tcscat if TCS catalog) 
	global ::$w_.tcs

	if {[set $w_.tcs $itk_option(-tcs)]} {
	    tcscat $w_.cat
	} else {
	    astrocat $w_.cat
	}

	# normally -catalog should be specified when creating this widget
	# if not, choose a default...
	if {"$itk_option(-catalog)" == ""} {
	    return
	    if {[catch {set catalog_list [$w_.cat info $itk_option(-catalogtype)]} msg]} {
		error_dialog $msg $w_
		return
	    }
	    set itk_option(-catalog) [lindex $catalog_list [expr [llength $catalog_list]-1]]
	}


	# open the catalog
	set name $itk_option(-catalog)
	if {[catch {$w_.cat open $name} msg]} {
	    error_dialog $msg $w_
	    return
	}
	
	# set iscat_ to true if the catalog is not an image server
	set iscat_ 1
	if {"[$w_.cat servtype]" == "imagesvr"} {
	    set iscat_ 0
	} 

	# if this is a local catalog, add it to the catalog menus and tree
	# XXX should check if local catalog was already known
	if {"[$w_.cat servtype]" == "local"} {
	    update_catalog_menus

	    # add to catalog tree, if there is one
	    catch {.catinf insert_node $name}
	    
	    # add an entry to the config file for this catalog
	    cat::CatalogInfo::save "" $w_ 0
	}
	
	# display catalog name in header and icon
	wm title $w_ "[$w_.cat longname $name] ($itk_option(-number))"
	wm iconname $w_ "[$w_.cat shortname $name]"
    }

    
    # add the search options panel

    protected method add_search_options {} {
	# vmAstroQuery(n) widget for displaying catalog search options.
	itk_component add searchopts {
	    set searchopts_ [::cat::vmAstroQuery $w_.searchopts \
				 -relief groove \
				 -borderwidth 2 \
				 -debug $itk_option(-debug) \
				 -astrocat [code $w_.cat] \
				 -searchcommand [code $this search] \
				 -feedbackcommand [code $this set_feedback] \
				 -command [code $this query_done]]
	}
	pack $itk_component(searchopts) \
	    -side top -fill x
    }

    
    # add the table for displaying the query results

    protected method add_result_table {} {
	# QueryResult(n) widget to display catalog query results
	itk_component add results {
	    set results_ [::cat::vmQueryResult $w_.results \
			      -astrocat [code $w_.cat] \
			      -title "Search Results" \
			      -hscroll 1 \
			      -height 12 \
			      -sortcommand [code $this set_sort_cols] \
			      -layoutcommand [code $this set_show_cols] \
			      -selectmode extended \
			      -exportselection 0]
	} {
	}
	pack $itk_component(results) -side top -fill both -expand 1
	bind $results_.listbox <ButtonRelease-1> [code $this select_result_row]
	$results_ set_options {MORE PREVIEW more preview} Show 0
    }

###################### New method for vmmps ########################
     public method arch { object } {
        global catname image2 mybind
        global myinfo myresults mythis
	global mode
       
        set myinfo $info_
        set myresults $results_
        set mythis $this
	set mode $object

	 if {$object=="object"} {
	     set mybind .skycat1.image.pick.mainf.topf.rectf.rf.zoomView.image.imagef.canvas
	     catch {file delete [file dirname $catname]/arcpoint.tmp}
	     utilReUseWidget rtd::vmRtdImagePick .skycat1.image.pick \
		 -target_image ::.skycat1.image \
		 -command "" \
		 -verbose 0 \
		 -orient Vertical \
		 -debug 0 \
		 -shorthelpwin ::.skycat1
	     .skycat1.image.pick pick_object
	     update
	     bind $mybind <ButtonRelease-1> {
		 exec echo $image2(X) $image2(Y) >> [file dirname $catname]/arcpoint.tmp
		 .skycat1.image.pick mark_spot $image2(X) $image2(Y) image7 $mybind 90 1 1
	     }
	 }
	
	 if {$object=="slit"} {
	     set mybind .skycat1.image.pick1.mainf.topf.rectf.rf.zoomView.image.imagef.canvas
	     catch {file delete [file dirname $catname]/arcpoint.tmp}
	     utilReUseWidget rtd::vmRtdImagePick .skycat1.image.pick1 \
		 -target_image ::.skycat1.image \
		 -command "" \
		 -verbose 0 \
		 -orient Vertical \
		 -debug 0 \
		 -shorthelpwin ::.skycat1
	     .skycat1.image.pick1 pick_object
	     update
	     bind $mybind <ButtonRelease-1> {
		 exec echo $image2(X) $image2(Y) >> [file dirname $catname]/arcpoint.tmp
		 .skycat1.image.pick1 mark_spot $image2(X) $image2(Y) image7 $mybind 90 1 1
	     }
	 }
    }

    public method vmmpsPickObject { object } {

        global catname image2 mybind
        global myinfo myresults mythis
	global mode 
	global rownum_for_newslit
	set rownum_for_newslit -1

        global mps_window
	global slit_window

        set myinfo $info_
        set myresults $results_
        set mythis $this
	set mode $object

	if {$object=="object"} {
	    set mybind .skycat1.image.pick2.mainf.topf.rectf.rf.zoomView.image.imagef.canvas
	    utilReUseWidget rtd::vmmpsImagePick .skycat1.image.pick2 \
		-target_image ::.skycat1.image \
		-command "" \
		-verbose 0 \
		-orient Vertical \
		-debug 0 \
		-shorthelpwin ::.skycat1
            .skycat1.image.pick2 pick_object
            update
            bind $mybind <ButtonRelease-1> {
		.skycat1.image.pick2 mark_spot $image2(X) $image2(Y) image7 $mybind 90 1 1
            }
	} 
	if {$object=="slit"} {
	  set rownum_for_newslit 0
           if {! [confirm_dialog "Overlaping of slits is NOT checked in the current version!.\n Do you want to continue?" .skycat1] } {
	       return
	   }

	    set mybind .skycat1.image.pick3.mainf.topf.rectf.rf.zoomView.image.imagef.canvas
	    utilReUseWidget rtd::vmmpsImagePick .skycat1.image.pick3 \
		-target_image ::.skycat1.image \
		-command "" \
		-verbose 0 \
		-orient Vertical \
		-debug 0 \
		-shorthelpwin ::.skycat1
            .skycat1.image.pick3 pick_object
            update
            bind $mybind <ButtonRelease-1> {
		.skycat1.image.pick3 mark_spot $image2(X) $image2(Y) image7 $mybind 90 1 1
            }
	} 
	
    }


###################### New method for vmmps ########################
     public method _ref { w_ } {
         set skycat "namespace inscope ::skycat::SkyCat .skycat1.image"
	 set image [$skycat get_image]
	 if {[$image isclear]} {
	     warning_dialog "No image is currently loaded" $w
	     return
	 }

	 if ![winfo exists $w_.pick] {
	     set sampleSize_ 40

	     # wait for user to click in image and check if operation
	     # has been 'cancelled'
	     set list [pick_object_in_image $sampleSize_ $sampleSize_ $w_]   
	     if ![lempty $list] {
		 global objRa objDec
		 set objRa [lindex $list 2]
		 set objDec [lindex $list 3]
		 #puts "new object is $objRa, $objDec"
	     }
	 }	    

     }


#------------------------------------------------------
# METHOD : pick_object_in_image
#
# this method is called to allow the user to pick an object in the
# main image.
# The arguments indicate the size of the area around the mouse ptr
# to examine.
# The return value is a list of:
# "ra dec equinox fwhmX fwhmY angle objectPeak meanBackground"
# as returned from the rtdimage "statistics" subcommand,
# or an error.
#------------------------------------------------------

public proc pick_object_in_image {width height w_} {

#    set target_canvas_ [$itk_option(-target_image) get_canvas]
#    $target_canvas_ create rectangle $x1 $y1 $x2 $y2 \
	-tags tcsRefStar -outline white

#    set cursor [$target_canvas_ cget -cursor]
#    $target_canvas_ configure -cursor cross

#    set bindtags [bindtags $target_canvas_]
    set tag pick$w_
    bind $tag <ButtonRelease-1> [code cat::vmAstroCat::picked_object]
#    bindtags $target_canvas_ $tag

    global ::$w_.picked
    set $w_.picked {}
    tkwait variable $w_.picked

#    bindtags $target_canvas_ $bindtags

#    $target_canvas_ configure -cursor $cursor
    set ret [set $w_.picked]

    if {"$itk_option(-command)" != ""} {
       eval $itk_option(-command) [list $ret]
    }

    return $ret
}

#------------------------------------------------------
# METHOD : picked_object
#  
# this method is called when the user clicks in the image to
# select an object or star for the "pick_object" method.
#------------------------------------------------------

public proc picked_object {} {

   # display busy cursor in image and pick window...
   $itk_option(-target_image) busy {
      busy {

        if {[catch {set list [$image_ statistics]} msg]} {
           error_dialog $msg .skycat1
           cancel_pick
        } else {
            picked_wcs_object $list
        }
      } ;# end busy
   }
}

    public proc cancel_pick {} {
        global ::$w_.picked
        set $w_.picked ""
    }

    public proc picked_wcs_object {retval} {
        global ::$w_.picked
        set $w_.picked $retval
    }



    # add the dialog button frame

######################### Modified for vmmps #########################
    protected method mps_add_dialog_buttons {} {
	# dialog buttons
	global catname mps_window

	#puts "DB> MPS window - $w_"
	set mps_window $w_

	# 1. AAG - A text line
	pack \
	[label $w_.label1 -text "Modify Flags:"\
	-font -Adobe-Times-Medium-R-Normal-*-180-*] \
	    -side top -padx 1m -pady 1m

	# 2. AAG - A Frame to put functions buttons in it
	pack [frame $w_.selbuttons -borderwidth 2 -relief raised] \
	    -side top -fill x

	# 3. AAG - The Functions button in frame
	pack \
	[button $w_.reference \
	-text "Ref" \
	-command [code $this _sel R]] \
        [button $w_.newref \
        -text "NewRef" \
        -command [code $this vmmpsPickObject "object"]] \
	[button $w_.compulsory \
	-text "Comp" \
	-command [code $this _sel C]] \
	[button $w_.forbidden \
	-text "Forb" \
	-command [code $this _sel F]] \
	[button $w_.arch \
	-text "Curved" \
	-command [code $this arch "object"]] \
	[button $w_.unselect \
	-text "UnSel" \
	-command [code $this _sel S]] \
	    -side left -expand 1 -pady 2m -in $w_.selbuttons

	 # 4. AAG. Last button
         pack \
	[button $w_.spoc \
	-text "Automatic Slit Assignment" \
	-command [code $this export]] \
	   -side left -expand 1 

	 add_short_help $w_.reference \
	    {{bitmap b1} = select a reference object}

	 add_short_help $w_.newref \
	    {{bitmap b1} = open pick-object window to allow add new reference stars to the catalog}

	 add_short_help $w_.compulsory \
	    {{bitmap b1} = select a compulsory object}

	 add_short_help $w_.forbidden \
	    {{bitmap b1} = select a forbidden object}

	 add_short_help $w_.arch \
	    {{bitmap b1} = open the curved slit definition panel}

	 add_short_help $w_.unselect \
	    {{bitmap b1} = unselect a reference/compulsory/forbidden object}

	 add_short_help $w_.spoc \
	    {{bitmap b1} = open the slit positioning and optimization panel}


    }

    protected method slit_add_dialog_buttons {} {
	# dialog buttons
	global mps_window
	global slit_window
	global Ncur

	#puts "DB> SLITS window - $w_"
	set slit_window $w_

	# AAG - close mps window here"
	#if {$mps_window != ""} {
	#    puts "DB> Closing MPS window $mps_window"
	#    $mps_window close
        #}
	
	set Nmask [cat::vmmps_spoc_gui::getNmask]
	#puts "DB> Number of Mask Layers: $Nmask"

        # LabelMenu(n) widget to choose scale factor
        itk_component add choose {
            set m [util::LabelMenu $w_.choose \
                       -text "Choose Mask:" \
                       -relief groove \
                       -labelfont $itk_option(-labelfont) \
                       -valuefont $itk_option(-valuefont) \
                       -labelwidth $itk_option(-labelwidth) \
                       -valuewidth 5 \
                       -anchor e]
        } {
            keep -state -background -foreground
        }
        pack \
	    [label $w_.label1 -text "Modify Slits:"\
		 -font -Adobe-Times-Medium-R-Normal-*-180-*] \
            -side top -padx 1m -pady 1m

        #pack $itk_component(choose) -side top -fill x -ipadx 0.5m -ipady 0.5m
        pack $itk_component(choose) -side top -padx 1m -pady 1m

        for {set n 1} {$n <= $Nmask} {incr n 1} {
		  $w_.choose add -label "$n" -command [code $this search_mask $n]
	}

	$itk_component(choose) config -value $Ncur

	 add_short_help $w_.choose \
		{{bitmap b1} = Displays the current assigned slits on the image}
	    

	pack [frame $w_.slitbuttons -borderwidth 2 -relief raised] \
	    -side top -fill x


	    pack \
		[button $w_.unselect \
		     -text "UnSel" \
		     -command [code $this remove_selected]] \
		[button $w_.newslit \
		     -text "NewSlit" \
		     -command [code $this vmmpsPickObject "slit"]] \
		[button $w_.newcurv \
		     -text "NewCurved" \
		     -state disabled \
		     -command [code $this arch "slit"]] \
		[button $w_.slits \
		     -text "Plot Slits" \
		     -command [code $this slits]] \
		[button $w_.slits_c \
		     -text "Clear Slits" \
		     -command [code $this clearslits]] \
		-side left -expand 1 -pady 2m -in $w_.slitbuttons


	    
	    add_short_help $w_.unselect \
		{{bitmap b1} = Un-select a slit from the slit of the current Mask}
	    
	    add_short_help $w_.newslit \
		{{bitmap b1} = Define a new slit in the list of the current Mask}
	    
	    add_short_help $w_.newcurv \
		{{bitmap b1} = open the curved slit definition panel}
	    
	    add_short_help $w_.slits \
		{{bitmap b1} = plot slits and spectrum overlay}
	    
	    add_short_help $w_.slits_c \
		{{bitmap b1} = Clear slits in overlay}
	    

	pack \
	    [button $w_.makeadp \
		 -text "Make Final $Nmask ADPs" \
		 -command [code $this save_adp $Nmask]] \
	    -side left -expand 1

	add_short_help $w_.makeadp \
	    {{bitmap b1} = Save All ADP files from current adp_cat catalogs}

    }

    protected method slitcat_add_dialog_buttons {} {
        # dialog buttons

	global slitcat_window

	#puts "DB> SLITCAT window - $w_"
	set slitcat_window $w_

        pack [frame $w_.slitbuttons -borderwidth 2 -relief raised] \
            -side top -fill x


        pack \
        [button $w_.slits \
        -text "Plot Slits" \
        -command [code $this slits]] \
        [button $w_.clearslits \
        -text "Clear Slits" \
        -command [code $this clearslits]] \
            -side left -expand 1 -pady 2m -in $w_.slitbuttons

         add_short_help $w_.slits \
            {{bitmap b1} = plot slits and spectrum overlay}

         add_short_help $w_.clearslits \
            {{bitmap b1} = clear slits and spectrum overlay}
    }


    public method save_adp {nmask} {
	global catname
	global catname_for_newslit
	global quad
	global slit_window
        global rootbase

	set path [file rootname $catname]
	set i [string last M $path]
	if {$i != -1} {
	    set relname [string range $path 0 [ expr $i - 1 ] ]
	}
	# AAG - Ask if save & convert the current adp_cat files
	set file2save [glob -nocomplain $relname\M*.adp]
	if {![confirm_dialog "The following files exist: \n $file2save \n Do you want to save them ?" $w_]} {
            return
	}
	#puts "DB> Saving $relname\M*.adp files"
	for {set n 1} {$n <= $nmask} {incr n 1} {
	    set name $relname\M$n\Q$quad.adp
	    #
	    # JLBQ Ask about of lenght of the adp file.
	    #
	    set adpnameaux $name
	    if {[string last "/" $adpnameaux] != -1} {
	    	set adpname [string range $adpnameaux [ expr [string last "/" $adpnameaux] + 1] [string length $adpnameaux]]
	    }
	    #puts "DB> adpnameaux = $adpnameaux length [string length $adpnameaux]."
	    #puts "Check the string length of adp file name $adpname length [string length $adpname]."
	    if {[string length $adpname] > 79} {
	         confirm_dialog "WARNIG: The lenght of file $adpname exede 79 characteres\n Could you change it before to continue."
	         return
	    }
	    if {[file exists $name\_cat]} {
	   	 conv_to_adp $name
	    }
	}
	
	#AAG - Ask if Proceed, Exit or Cancel
	# Clean if Proceed
	set choice [choice_dialog \
			"What do you want to do now?:" \
			{{Next Quadrant} {Exit VMMPS} Cancel} {Cancel} $w_]

	if {$choice == "Cancel"} {
	    return
	} elseif {$choice == "Next Quadrant"} {
	    clearslits
	    clearsymbol
	    $slit_window close
	    .skycat1.image clear
            catch {eval "file delete [glob -nocomplain [file dirname $rootbase]/*.adp_cat]"}
            #puts "DB> [file dirname $rootbase]/*.adp_cat removed"
            #catch {eval "file delete [glob -nocomplain [file dirname $rootbase]/*.dat]"}
            #puts "DB> [file dirname $rootbase]/*.dat removed"	    
	    #puts "Removing [file rootname $catname_for_newslit].dat"
	    catch {eval "file delete [file rootname $catname_for_newslit].dat"}
	    catch {eval "file delete $catname_for_newslit.BAK"}
	    #catch {eval "file delete $catname_for_newslit"}
            #catch {eval "file delete [glob -nocomplain [file dirname $rootbase]/*.tmp]"}
	    #puts "DB> [file dirname $rootbase]/*.tmp removed"
            #catch {eval "file delete [glob -nocomplain [file dirname $rootbase]/*.BAK]"}
	    #puts "DB> [file dirname $rootbase]/*.BAK removed"

	} else {
	    ExitApp
	}

    }

    public method search_mask { mask } {
	global catname
	global slit_window
	global Ncur
	global quad

	clearsymbol
	clearslits
	set Ncur $mask
	set slit_window $w_
	# error_dialog "This is not ready yet....!!!"
	#puts "DB> Current loaded catalog: $catname"
        set id "namespace inscope ::skycat::SkyCat .skycat1.image"
        set classname "::skycat::vmSkySearch"
        set debug 0
	set catadp  [file rootname $catname]
	set i [string last M $catadp]
	if {$i != -1} {
	    set catadp [string range $catadp 0 [ expr $i - 1 ] ]
	}
        set catadp $catadp\M$mask\Q$quad.adp
	if {[file exists $catadp\_cat]} {
		#puts "DB> File $catadp\_cat exist. Keeping"
	} else {
		#puts "DB> File $catadp\_cat DOES NOT exist. Converting from current adp."
		cat::vmAstroCat::conv_from_adp $catadp
	}
        set catname $catadp\_cat
	#puts "DB> Loading new catalog: $catname"

	$slit_window close
	cat::vmAstroCat::new_catalog $catname $id $classname $debug [cat::vmAstroCat::is_tcs_catalog $catname] "catalog"
	
	set Ncur $mask
	slits
	return

    }
   



    protected method add_dialog_buttons {} {
	# dialog buttons

	pack [frame $w_.buttons -borderwidth 2 -relief raised] \
	    -side top -fill x
	
	if {$iscat_} {
	    set search_label "Search"
	} else {
	    set search_label "Get Image"
	}
	

	pack \
	    [button $w_.search \
		 -text $search_label \
		 -command [code $this search]] \
	    -side left -expand 1 -pady 2m -in $w_.buttons

	if {$iscat_} {
	    pack \
		[button $w_.more \
		     -text "More Info" \
		     -state disabled \
		     -command [code $this more]] \
		[button $w_.preview \
		     -text "Preview" \
		     -state disabled \
		     -command [code $this preview]] \
		-side left -expand 1 -pady 2m -in $w_.buttons
	}

	pack \
	    [button $w_.stop \
		 -text "Stop" \
		 -state disabled \
		 -command [code $this interrupt]] \
	    [button $w_.close \
		 -text "Close" \
		 -command [code $this close]] \
	    -side left -expand 1 -pady 2m -in $w_.buttons

	# add a help button, if there is a help URL
	if {"[$w_.cat help]" != ""} {
	    pack \
		[button $w_.help \
		     -text "Help" \
		     -command [code $this help]] \
		-side left -expand 1 -pady 2m -in $w_.buttons -before $w_.close 
	}
    }

 ######################### New method for vmmps #######################
    public proc conv_from_adp {name} {
       set adpname $name
       #puts "DB> Converting adp $name in skycat format"
       set catalogname $name\_cat
       set spectlen ""
       catch {set spectlen [exec grep "INS.ADF.GRISM.SPECTLEN" $adpname | cut -f2 -d\"]}
       if {$spectlen == ""} {
              #puts "DB> INS.ADF.GRISM.SPECTLEN not found"
	      error_dialog "The ADP File does not seem to have the right format. Please check" .skycat1
              return -code return
       } 
       
       set cat [open $catalogname w]
       puts $cat "QueryResult"
       puts $cat ""
       puts $cat "# Config entry for original catalog server:"
       puts $cat "serv_type: local"
       puts $cat "long_name: $catalogname"
       puts $cat "short_name: [file tail $catalogname]"
       puts $cat "url: $catalogname"
       puts $cat "symbol: {ALPHA_VIMOS DELTA_VIMOS} {square yellow {} {} {} {}} {15 {}}"
       puts $cat "# Spectrum _Length: $spectlen"
       puts $cat "# End config entry"
       puts $cat ""
       puts $cat "ID	ALPHA_VIMOS	DELTA_VIMOS	X_SLIT	Y_SLIT	DIMX_SLIT	DIMY_SLIT"
       puts $cat "------	-----------	-----------	------	------	---------	---------"
       ::close $cat


       catch {exec grep "INS\\.REF" $adpname | grep "\\.ID " | cut -d\" -f2 > $adpname\_id}
       catch {exec grep "INS\\.REF" $adpname | grep "\\.OBJ\\.RA " | cut -d\" -f2 > $adpname\_ra}
       catch {exec grep "INS\\.REF" $adpname | grep "\\.OBJ\\.DEC " | cut -d\" -f2 > $adpname\_dec}
       catch {exec grep "INS\\.REF" $adpname | grep "\\.X " | cut -d\" -f2 > $adpname\_x}
       catch {exec grep "INS\\.REF" $adpname | grep "\\.Y " | cut -d\" -f2 > $adpname\_y}
       catch {exec grep "INS\\.REF" $adpname | grep "\\.DIMX " | cut -d\" -f2 > $adpname\_dimx}
       catch {exec grep "INS\\.REF" $adpname | grep "\\.DIMY " | cut -d\" -f2 > $adpname\_dimy}

       catch {exec grep "INS\\.SLIT" $adpname | grep "\\.ID " | cut -d\" -f2 >> $adpname\_id}
       catch {exec grep "INS\\.SLIT" $adpname | grep "\\.OBJ\\.RA " | cut -d\" -f2 >> $adpname\_ra}
       catch {exec grep "INS\\.SLIT" $adpname | grep "\\.OBJ\\.DEC " | cut -d\" -f2 >> $adpname\_dec}
       catch {exec grep "INS\\.SLIT" $adpname | grep "\\.X " | cut -d\" -f2 >> $adpname\_x}
       catch {exec grep "INS\\.SLIT" $adpname | grep "\\.Y " | cut -d\" -f2 >> $adpname\_y}
       catch {exec grep "INS\\.SLIT" $adpname | grep "\\.DIMX " | cut -d\" -f2 >> $adpname\_dimx}
       catch {exec grep "INS\\.SLIT" $adpname | grep "\\.DIMY " | cut -d\" -f2 >> $adpname\_dimy}
       catch {exec paste $adpname\_id $adpname\_ra $adpname\_dec $adpname\_x $adpname\_y $adpname\_dimx $adpname\_dimy >> $catalogname}

       catch {file delete $adpname\_id}
       catch {file delete $adpname\_ra}
       catch {file delete $adpname\_dec}
       catch {file delete $adpname\_x}
       catch {file delete $adpname\_y}
       catch {file delete $adpname\_dimx}
       catch {file delete $adpname\_dimy}
    }

 ######################### New method for vmmps #######################
    public proc conv_to_adp { name } {
	global ::env

	set catalogname $name\_cat
	#puts "DB> Converting catalog $catalogname in paf format ($name)"
	set adpname $name
	set spectlen [exec grep "Length" $catalogname | cut -f2 -d\:]
       
        if {[file exists $catalogname] && [file readable $catalogname]} {
            #puts "DB> Opening $catalogname for reading"
            set fId [::open $catalogname r ]
            set index(ID) -1
	    set index(ALPHA) -1
	    set index(DELTA) -1
	    set index(X) -1
	    set index(Y) -1
	    set index(DX) -1
	    set index(DY) -1
            set f_header 0
	    set N_slit 0

            while {[gets $fId line] >= 0} {
                if {!$f_header} {
                    if {[string match *ID* $line]} {
                        set index(ID) [lsearch -exact $line ID]
			set index(ALPHA) [lsearch -exact $line ALPHA_VIMOS]
			set index(DELTA) [lsearch -exact $line DELTA_VIMOS]
			set index(X) [lsearch -exact $line X_SLIT]
			set index(Y) [lsearch -exact $line Y_SLIT]
			set index(DX) [lsearch -exact $line DIMX_SLIT]
			set index(DY) [lsearch -exact $line DIMY_SLIT]
                    }
                    if {[string match *---* $line]} {
                        set f_header 1
                    }
                } else {
		    set N_slit [expr $N_slit + 1]
		    foreach var "ID ALPHA DELTA X Y DX DY" {
			if {$index($var) != -1} {
			    set val($N_slit,$var) [lindex $line $index($var)]
			}		
		    }
		    
                }
            }
            ::close $fId
        } else {
            puts stdout "Warning:$catalogname: does not exist or has no read permission"
	    return
        }
	set buf ""
	catch {set buf [exec grep -n "SLIT1.TYPE" $adpname]}
	if {$buf == ""} {
		#puts "DB> no slits were found in the original $adpname"
		exec cp $adpname $adpname\_head
	} else {
	  set headlen [exec echo $buf | cut -d: -f1]
	  set headlen [expr $headlen - 1]
	  exec head -n $headlen $adpname > $adpname\_head
	  #puts "DB> Number of slits found in $adpname: $N_slit"
	}

	set buf ""
	catch {set buf [exec grep -n "CURVE" $adpname]}
	if {$buf == ""} {
		#puts "DB> no curve slits were found in the original $adpname"
		exec touch $adpname\_tail
	} else {
	  set taillen [lindex [exec echo $buf | cut -d: -f1] 0]
	  set taillen [expr $taillen - 1]
	  set total_lines [lindex [exec wc -l $adpname] 0]
	  set taillen [expr $total_lines - $taillen]
	  exec tail -$taillen $adpname > $adpname\_tail
	}

        set fadp_h [::open $adpname\_head r ]
	set N_ref 0
	while {[gets $fadp_h line] >= 0} {
	      if {[string match *REF*ID* $line]} {
		set N_ref [expr $N_ref + 1]
		set ref_id($N_ref) [lindex $line 1]
	      }
	}
	::close $fadp_h

        set fadp_t [::open $adpname\_tail r ]
	set N_curv 0
        set c 0
	while {[gets $fadp_t line] >= 0} {
	      if {[string match *TYPE*CURVE* $line]} {
		set N_curv [expr $N_curv + 1]
	        set c 1
	      }
	      if {[string match *SLIT*ID* $line] && $c} {
		set curv_id($N_curv) [lindex $line 1]
		set c 0
		}
	}
	::close $fadp_t

	set x 0
	for {set j 1} {$j <= $N_ref} {incr j 1} {
	   for {set n 1} {$n <= $N_slit} {incr n 1} {
	     if {$val($n,ID) == $ref_id($j)} {
		set x [expr $x + 1]
		foreach var "TYPE ID OBJ.RA OBJ.DEC X Y DIMX DIMY" {
		   set line [exec grep INS.REF$j.$var $adpname\_head] 
		   set val_ref($x,$var) [lindex $line 1]
		}
		break
	     } else {
		#puts "Ref Id: $ref_id($j) not found in slit"
	     }
	   }
	}

	for {set n 1} {$n <= $N_ref} {incr n 1} {
           exec cat $adpname\_head | grep -v REF$n > $adpname\_head2
	   catch {file rename -force $adpname\_head2 $adpname\_head}
	   }

 	#puts "DB> Found $x ref slit from originall $N_ref"	
	set N_ref_orig $N_ref
	set N_ref $x

	if {$N_ref > 0} {

	   set fref [::open $adpname\_ref w ]
	   for {set n 1} {$n <= $N_ref} {incr n 1} {

	        puts $fref "INS.REF${n}.TYPE \"$val_ref($n,TYPE)\""
	        puts $fref "INS.REF${n}.ID \"$val_ref($n,ID)\""
	        puts $fref "INS.REF${n}.OBJ.RA \"$val_ref($n,OBJ.RA)\""
	        puts $fref "INS.REF${n}.OBJ.DEC \"$val_ref($n,OBJ.DEC)\""
	        puts $fref "INS.REF${n}.X \"$val_ref($n,X)\""
	        puts $fref "INS.REF${n}.Y \"$val_ref($n,Y)\""
	        puts $fref "INS.REF${n}.DIMX \"$val_ref($n,DIMX)\""
	        puts $fref "INS.REF${n}.DIMY \"$val_ref($n,DIMY)\""


	   }
	   ::close $fref
	} else {
	   exec touch $adpname\_ref
	}

	set fslit [::open $adpname\_slit w ]
	set x 1 
	for {set n 1} {$n <= $N_slit} {incr n 1} {
	  set flag 0
	  for {set j 1} {$j <= $N_ref_orig} {incr j 1} {
	     if {$val($n,ID) == $ref_id($j)} {
		set flag 1
		break
	     }
	  } 

	  set flag_curv 0
	  for {set j 1} {$j <= $N_curv} {incr j 1} {
	     if {$val($n,ID) == $curv_id($j)} {
		set flag_curv 1
		break
	     }
	  } 
	  if {!$flag && !$flag_curv} {
	     puts $fslit "INS.SLIT${x}.TYPE \"RECTANGLE\""
	     puts $fslit "INS.SLIT${x}.ID \"$val($n,ID)\""
	     puts $fslit "INS.SLIT${x}.OBJ.RA \"$val($n,ALPHA)\""
	     puts $fslit "INS.SLIT${x}.OBJ.DEC \"$val($n,DELTA)\""
	     puts $fslit "INS.SLIT${x}.X \"$val($n,X)\""
	     puts $fslit "INS.SLIT${x}.Y \"$val($n,Y)\""
	     puts $fslit "INS.SLIT${x}.DIMX \"$val($n,DX)\""
	     puts $fslit "INS.SLIT${x}.DIMY \"$val($n,DY)\""
	     set x [expr $x + 1]
	  }

	}
	::close $fslit

	set first_curv $x

	set z 0
        set line2 [exec head -n 1 $adpname\_tail]
        set t [scan $line2 {INS.SLIT%d.TYPE*} f2]
	for {set j 1} {$j <= $N_curv} {incr j 1} {
	   for {set n 1} {$n <= $N_slit} {incr n 1} {
	     if {$val($n,ID) == $curv_id($j)} {
		set z [expr $z + 1]
		foreach var "TYPE ID OBJ.RA OBJ.DEC X Y DIMX DIMY BEZIER.DY BEZIER.AX BEZIER.BX BEZIER.CX BEZIER.XX BEZIER.AY BEZIER.BY BEZIER.CY BEZIER.YY" {
		    set line ""
		    catch {set line [exec grep INS.SLIT$f2.$var $adpname\_tail]}
		    if {$line ==""} {
		       puts "Error"
		   }
		   set val_curv($z,$var) [lindex $line 1]
		}
		break
	     } else {
		#puts "Curv Id: $ref_id($j) not found in slit"
	     }
	   }
	   set f2 [expr $f2 + 1]
	}

 	#puts "DB> Found $z curv slit from original $N_curv"	
	set N_curv $z

	set f $first_curv
	if {$N_curv > 0} {

	   set fcurv [::open $adpname\_curv w ]
	   for {set n 1} {$n <= $N_curv} {incr n 1} {

	        puts $fcurv "INS.SLIT${f}.TYPE \"$val_curv($n,TYPE)\""
	        puts $fcurv "INS.SLIT${f}.ID \"$val_curv($n,ID)\""
	        puts $fcurv "INS.SLIT${f}.OBJ.RA \"$val_curv($n,OBJ.RA)\""
	        puts $fcurv "INS.SLIT${f}.OBJ.DEC \"$val_curv($n,OBJ.DEC)\""
	        puts $fcurv "INS.SLIT${f}.X \"$val_curv($n,X)\""
	        puts $fcurv "INS.SLIT${f}.Y \"$val_curv($n,Y)\""
	        puts $fcurv "INS.SLIT${f}.DIMX \"$val_curv($n,DIMX)\""
	        puts $fcurv "INS.SLIT${f}.DIMY \"$val_curv($n,DIMY)\""
	        puts $fcurv "INS.SLIT${f}.BEZIER.DY \"$val_curv($n,BEZIER.DY)\""
	        puts $fcurv "INS.SLIT${f}.BEZIER.AX \"$val_curv($n,BEZIER.AX)\""
	        puts $fcurv "INS.SLIT${f}.BEZIER.BX \"$val_curv($n,BEZIER.BX)\""
	        puts $fcurv "INS.SLIT${f}.BEZIER.CX \"$val_curv($n,BEZIER.CX)\""
	        puts $fcurv "INS.SLIT${f}.BEZIER.XX \"$val_curv($n,BEZIER.XX)\""
	        puts $fcurv "INS.SLIT${f}.BEZIER.AY \"$val_curv($n,BEZIER.AY)\""
	        puts $fcurv "INS.SLIT${f}.BEZIER.BY \"$val_curv($n,BEZIER.BY)\""
	        puts $fcurv "INS.SLIT${f}.BEZIER.CY \"$val_curv($n,BEZIER.CY)\""
	        puts $fcurv "INS.SLIT${f}.BEZIER.YY \"$val_curv($n,BEZIER.YY)\""
		set f [expr $f + 1]

	   }
	   ::close $fcurv
	} else {
	   exec touch $adpname\_curv
	}

	set total_slit_num [expr $f - 1]
	# --


	exec cat $adpname\_head $adpname\_ref $adpname\_slit $adpname\_curv > $adpname
	catch {file delete $adpname\_head}
	catch {file delete $adpname\_slit}
	catch {file delete $adpname\_ref}
	catch {file delete $adpname\_curv}
	catch {file delete $adpname\_tail}


        set fadp [::open $adpname r ]
	set fadp_ [::open $adpname\_ w ]
	while {[gets $fadp line] >= 0} {
	   if {[string match *SLIT.NO* $line]} {
	      puts $fadp_ "INS.SLIT.NO \"$total_slit_num\""
	   } else {
	      if {[string match *REF.NO* $line]} {
		 puts $fadp_ "INS.REF.NO \"$N_ref\""
	      } else {
		 if {[string match *ARC.NO* $line]} {
		     puts $fadp_ "INS.ARC.NO \"$N_curv\""
		 } else {
	            puts $fadp_ $line 
		 }
	      }
	   }
        }
	::close $fadp
	::close $fadp_

	# MKI - 2004/05/16
	# Include checksum obtained from CRC32 binary.
	# It should be the last operation on this file.
	set crc [format "%s/bin/crc32" $env(VMMPS)] 
	if {[file exists $crc]} {
	    set crcoutput ""
	    catch {set crcoutput [exec $crc $adpname\_]}
	    if {$crcoutput == ""} {
		error "CRC32 wrong checksum calculation"
	    } else {
		set checksum [lindex $crcoutput 1]
		set fadp_ [::open $adpname\_ r ]
		set fadp_checksum [::open $adpname\_checksum w ]
		while {[gets $fadp_ line] >= 0} {
		    if {[string match *PAF.CHCK.CHECKSUM* $line]} {
			puts $fadp_checksum "PAF.CHCK.CHECKSUM \"$checksum\""
		    } else {		
			puts $fadp_checksum $line 				    
		    }
		}
	    ::close $fadp_
	    ::close $fadp_checksum
	    catch {file rename -force $adpname\_checksum $adpname}
	    catch {file delete $adpname\_}
	    }
	} else {
	    error "CRC32 not found !"
	}
	puts "JLBQ >> ..........  adpname = $adpname"
    }


 ######################### New method for vmmps #######################
    public method conv {name} {

       #puts "DB> Re-formating $name"
       set target_image_ image2

       set pos {}
       set radius {0.0 10.0}
       set nrows {10000}
       set searchcols 0
       set minvalues 0
       set maxvalues 0
       set nsearchcols 0
       set pi 3.141592654
	
       set buff ""

       set buff [eval exec vmmps_sel2 $name $pos $radius $nrows $nsearchcols $searchcols $minvalues $maxvalues]

       set nobj [llength $buff]
       set nobj_no 0

       #set heading [::cat::.vmastrocat headings]
       set heading [exec grep -i X_IMAGE $name | grep -i Y_IMAGE]
       set X_IMAGE [lsearch $heading X_IMAGE]
       set Y_IMAGE [lsearch $heading Y_IMAGE]

       # AAG- Look if FLAG is already present
       set FLAG [lsearch $heading FLAG]
       # AAG - Check if ALPHA_VIMOS and DELTA_VIMOS are present
       set ALPHA_VIMOS [lsearch $heading ALPHA_VIMOS]
       set DELTA_VIMOS [lsearch $heading DELTA_VIMOS]
       
       if {$ALPHA_VIMOS == -1 && $DELTA_VIMOS == -1} {
	   #puts "DB> ALPHA_VIMOS & DELTA_VIMOS are not present in the original VIMOS catalog. Adding ..."
	   set newwcs [open $name\_newwcs w]
	   puts $newwcs "ALPHA_VIMOS	DELTA_VIMOS"
	   puts $newwcs "-----------	-----------"
	   
	   for {set n 0} {$n < $nobj} {incr n 1} {
	       set row [lindex $buff $n]
	       set x [lindex $row $X_IMAGE]
	       set y [lindex $row $Y_IMAGE]
	       if {[catch {$target_image_ convert coords $x $y image ra dec wcs}]} {
		   set ra "outfield"
		   set dec "outfield"
		   set nobj_no [expr $nobj_no + 1]
	       }
	       puts $newwcs "$ra	$dec"
	   }
	   ::close $newwcs
	   
	   set buffer [exec grep -n "\\--" $name]
	   set headlen [exec echo $buffer | cut -d: -f1]
	   set headlen [expr $headlen - 2]
	   
	   set head [open $name\_head w]
	   for {set n 1} {$n <= $headlen} {incr n 1} {
	       puts $head ""
	   }
	   ::close $head
	   set headlen [expr $headlen + 1]
     
	   # AAG - We want to put FLAG column at the end
	   #puts "DB> Puting FLAG column at the end"
	   if {$FLAG != -1} {
	      set index1 $FLAG
	      set indexf [expr $FLAG + 1]
	      set index2 [expr $FLAG + 2]
	      exec cut -f$indexf $name > $name\_flag
	      if {$FLAG != 0} {
	        exec cut -f-$index1\,$index2\- $name > $name\_noflag
	      } else {
	        exec cut -f$index2\- $name > $name\_noflag
	      }
	      exec cut -f1 $name\_noflag > $name\_id
	      exec tail +$headlen $name\_noflag | cut -f2- > $name\_tmp
	      exec cat $name\_head $name\_tmp > $name\_nowcs       
	      exec cat $name\_head $name\_newwcs > $name\_tmp1
	      exec paste $name\_id $name\_tmp1 $name\_nowcs > $name\_tmp2

	      # The following is to clean the first lines in the 
	      # header, to be used properly after

	      set flagline ""
	      set flagfile [open $name\_flag r]
	      set flagfile2 [open $name\_flag2 w]
	      set n 0
	      set numline [expr $headlen - 1]
	      while {[gets $flagfile flagline] >= 0} {
		  if {$n < $numline} {
		    puts $flagfile2 ""
		    set n [expr $n + 1]
		  } else {
		    puts $flagfile2 [string toupper $flagline]
		  }
	      }
	      ::close $flagfile
	      ::close $flagfile2

	   } else {
	      #puts "DB> FLAG is not present"
	      exec cut -f1 $name > $name\_id
	      exec tail +$headlen $name | cut -f2- > $name\_tmp
	      exec cat $name\_head $name\_tmp > $name\_nowcs       
	      exec cat $name\_head $name\_newwcs > $name\_tmp1
	      exec paste $name\_id $name\_tmp1 $name\_nowcs > $name\_tmp2
	   }

	   catch {file delete $name\_id}
	   catch {file delete $name\_tmp}
	   catch {file delete $name\_tmp1}
	   catch {file delete $name\_nowcs}
	   catch {file delete $name\_newwcs}

	   if {$FLAG != -1} {
	      catch {file delete $name\_noflag}
	   }
       }
      
       set mpscol ""
       catch {set mpscol [exec grep -i X_RADIUS $name\_tmp2]}

       if {$mpscol == ""} {
	   #puts "DB> NO X_RADIUS"
	   
	   set image "no"
	   set world "no"
	   
	   set type ""
	   catch {set type [exec grep -i A_IMAGE $name\_tmp2 | grep -i B_IMAGE | grep -i THETA_IMAGE]}
	   if {$type != ""} {set image "yes"}
	   
	   set type ""
	   catch {set type [exec grep -i A_WORLD $name\_tmp2 | grep -i WORLD | grep -i THETA_WORLD]}
	   if {$type != ""} {set world "yes"}
	   
	   if {$image == "yes"} {
	       #puts "DB> A_IMAGE, B_IMAGE, THETA_IMAGE OK"
	       set A_IMAGE [lsearch $heading A_IMAGE]
	       set B_IMAGE [lsearch $heading B_IMAGE]
	       set THETA_IMAGE [lsearch $heading THETA_IMAGE]
        
	       set dim [open $name\_dim w]

	       if {$FLAG == -1} {
	           #puts "DB> X_RADIUS and FLAG are not present. Adding ..."   
		   puts $dim "X_RADIUS	FLAG"
		   puts $dim "----    ----"
	       } else {
		   #puts "FLAG is present. X_RADIUS is being calculated ..."
		   puts $dim "X_RADIUS"
		   puts $dim "----"
	       }

	       for {set n 0} {$n < $nobj} {incr n 1} {
		   set row [lindex $buff $n]
		   set A [lindex $row $A_IMAGE]
		   set B [lindex $row $B_IMAGE]
		   set THETA [lindex $row $THETA_IMAGE]
		   
		   set A [expr abs($A*cos($THETA*$pi/180.0))]
		   set AX $A
		   set BX $B
		   
		   if {$AX >= $BX} {set dX $AX} else {set dX $BX}
		   set dX [format "%.2f" $dX]
		   if {$FLAG == -1} {
		       puts $dim "$dX	S"
		   } else {
		      puts $dim "$dX"
		   }
	       }
	       ::close $dim
	   }
	     
	   if {$image == "no" && $world == "yes"} {
	       
	       set A_WORLD [lsearch $heading A_WORLD]
	       set B_WORLD [lsearch $heading B_WORLD]
	       set THETA_WORLD [lsearch $heading THETA_WORLD]
	       
	       set dim [open $name\_dim w]

	       if {$FLAG == -1} {
	           #puts "DB> X_RADIUS and FLAG are not present. Adding ..."   
		   puts $dim "X_RADIUS	FLAG"
		   puts $dim "----    ----"
	       } else {
		   #puts "DB> FLAG is present. Adding X_RADIUS ..."
		   puts $dim "X_RADIUS"
		   puts $dim "----"
	       }
	       
	       for {set n 0} {$n < $nobj} {incr n 1} {
		   set row [lindex $buff $n]
		   set A [lindex $row $A_WORLD]
		   set B [lindex $row $B_WORLD]
		   set THETA [lindex $row $THETA_WORLD]
             
		   set A [expr abs($A*sin($THETA*$pi/180.0))]
		   set dist [$target_image_ convert dist $A 0 deg AX AY image]
		   set dist [$target_image_ convert dist $B 0 deg BX BY image]
		   
		   if {$AX >= $BX} {set dX $AX} else {set dX $BX}
		   set dX [format "%.2f" $dX]
		   if {$FLAG == -1} {
		       puts $dim "$dX	S"
		   } else {
		       puts $dim "$dX"
		   }
		   
	       }
	       ::close $dim
	   }

	   if {$image == "no" && $world == "no"} {

	       set dim [open $name\_dim w]

	       if {$FLAG == -1} {
	           #puts "DB> X_RADIUS and FLAG are not present. Adding ..."   
		   puts $dim "X_RADIUS	FLAG"
		   puts $dim "----    ----"
	       } else {
		   #puts "DB> FLAG is present. Adding X_RADIUS ..."
		   puts $dim "X_RADIUS"
		   puts $dim "----"
	       }
	       
	       for {set n 0} {$n < $nobj} {incr n 1} {
		   if {$FLAG == -1} {
		       puts $dim "10	S"
		   } else {
		       puts $dim "10"
		   }
	       }
	       ::close $dim
	   }
	   
	   exec cat $name\_head $name\_dim > $name\_tmp1
	   if {$FLAG != -1} {
	       exec paste $name\_tmp2 $name\_tmp1 $name\_flag2 > $name\_tmp3
	   } else {
	       #puts "DB> Pasting with no FLAG"
	       exec paste $name\_tmp2 $name\_tmp1 > $name\_tmp3
	   } 
       } else {

	   #puts "DB> YES X_RADIUS" 

	   set image "no"
	   set world "no"
	   
	   set type ""
	   catch {set type [exec grep -i A_IMAGE $name\_tmp2 | grep -i B_IMAGE | grep -i THETA_IMAGE]}
	   if {$type != ""} {set image "yes"}
	   
	   set type ""
	   catch {set type [exec grep -i A_WORLD $name\_tmp2 | grep -i WORLD | grep -i THETA_WORLD]}
	   if {$type != ""} {set world "yes"}
	   
	   if {$image == "yes"} {
	       #puts "DB> A_IMAGE, B_IMAGE, THETA_IMAGE OK"
	       set A_IMAGE [lsearch $heading A_IMAGE]
	       set B_IMAGE [lsearch $heading B_IMAGE]
	       set THETA_IMAGE [lsearch $heading THETA_IMAGE]
        

	       if {$FLAG == -1} {
	           #puts "DB> FLAG is not present. Adding ..."   

	           set dim [open $name\_dim w]
		   puts $dim "FLAG"
		   puts $dim "----"
	           for {set n 0} {$n < $nobj} {incr n 1} {
		       puts $dim "S"
	           }
	           ::close $dim

	        } 
	   }
	     
	   if {$image == "no" && $world == "yes"} {
	       
	       set A_WORLD [lsearch $heading A_WORLD]
	       set B_WORLD [lsearch $heading B_WORLD]
	       set THETA_WORLD [lsearch $heading THETA_WORLD]
	       

	       if {$FLAG == -1} {
	           #puts "DB>  FLAG is not present. Adding ..."   
	           set dim [open $name\_dim w]
		   puts $dim "FLAG"
		   puts $dim "----"
	           for {set n 0} {$n < $nobj} {incr n 1} {
		       puts $dim "S"
		   }
	           ::close $dim

	       } 
	       
	   }

	   if {$image == "no" && $world == "no"} {


	       if {$FLAG == -1} {
	           #puts "DB> FLAG is not present. Adding ..."   
	           set dim [open $name\_dim w]
		   puts $dim "FLAG"
		   puts $dim "----"
	           for {set n 0} {$n < $nobj} {incr n 1} {
		       puts $dim "S"
		   } 
	           ::close $dim
	       }
	       
	   }
	   
	   if {$FLAG != -1} {
	       exec paste $name\_tmp2 $name\_flag2 > $name\_tmp3
	   } else {
	       #puts "DB> Pasting with no FLAG"
	       exec cat $name\_head $name\_dim > $name\_tmp1
	       exec paste $name\_tmp2 $name\_tmp1 > $name\_tmp3
	   } 

       }

       exec grep -v outfield $name\_tmp3 > $name
       catch {file delete $name\_dim}
       catch {file delete $name\_head}
       catch {file delete $name\_tmp1}
       catch {file delete $name\_tmp2}
       catch {file delete $name\_tmp3}

       if {$FLAG != -1 } {
          catch {file delete $name\_flag}
          catch {file delete $name\_flag2}
	}
   }
	
	public proc coord_format {alpha delta} {
	    if {$alpha == "" || $delta == ""} {
		set alpha 0
		set delta 0
	    }

        set alpha [expr $alpha / 15.0]
        set alphah [expr int($alpha)]
        set alpham [expr int(($alpha - $alphah) * 60)]
        set alphas [expr int(((($alpha - $alphah) * 60) - $alpham) * 60)]
	set alphadec [expr ((($alpha - $alphah) * 60) - $alpham) * 60 - $alphas]
	set alphadecInt [expr int($alphadec*100000)]
	set alpha [format %0.2d $alphah][format %0.2d $alpham][format %0.2d $alphas].[format %.5d $alphadecInt]

        if {$delta >= 0} {set sign ""} else {set sign "-"}
        set delta [expr abs($delta)]
        set deltad [expr int($delta)]
        set deltam [expr int(($delta - $deltad) * 60)]
        set deltas [expr int(((($delta - $deltad) * 60) - $deltam) * 60)]
        set deltadec [expr ((($delta - $deltad) * 60) - $deltam) * 60 - $deltas]
        set deltadecInt [expr int($deltadec*100000)]
        set delta [format %s $sign][format %0.2d $deltad][format %0.2d $deltam][format %0.2d $deltas].[format %.5d $deltadecInt]

        set coords $alpha
        lappend coords $delta
        return $coords
    }
    
    public proc symbol {name} {
	global ::skycat_images
       
	set buffer [exec grep -i -n "X_IMAGE" $name]
	set headpos [exec echo $buffer | cut -d: -f1]
	if {$headpos !=1} {
	  error_dialog "Catalog Format is wrong (columns names must be in first line)" .skycat1
	  return -code return
        }

	set buffer2 ""
	catch {set buffer2 [exec grep -n "\\--" $name]}
	if {$buffer2!=""} {
	    error_dialog "Catalog Format is wrong (remove \--- in your catalog)" .skycat1
	    return -code return

	}
	#set skycat "namespace inscope ::skycat::SkyCat .skycat1.image"
	#set testimage_ [$skycat get_image]
        #set imaname [$testimage_ cget -file]   

	set imaname [[lindex $skycat_images 0]  cget -file]
	puts "NAME=$imaname"
	set skyimage_ [[lindex $skycat_images 0] get_image]
	#set img [$pick_obj cget -target_image]
	#set skyimage_ [$img get_image]
        set instrume [$skyimage_ fits get INSTRUME]
	#puts "DATAMIN=[$skyimage_ fits get DATAMIN]"
	#puts "OBSERVE=[$skyimage_ fits get OBSERVER]"
	puts "INSTRUMENT=$instrume"
        if {$instrume != "VIMOS"} {
          error_dialog "No VIMOS image loaded!\nINSTRUME=$instrume" .skycat1
          return -code return
        }
        
        set naxis1 [image2 fits get "NAXIS1"] 
        set naxis2 [image2 fits get "NAXIS2"]
        if {$naxis1!=2048 || $naxis2!=2440} {
          error_dialog "FITS FILE: Wrong image dimensions\nNAXIS1=$naxis1\nNAXIS2=$naxis2" .skycat1
          return -code return
        }
        
        set quad [image2 fits get "HIERARCH ESO OCS CON QUAD"]
        if {$quad==""} {
          error_dialog "FITS FILE: No value found for keyword: HIERARCH ESO OCS CON QUAD" .skycat1
          return -code return
        }
        
        set progid [image2 fits get "HIERARCH ESO OBS PROG ID"]
        if {$progid==""} {
          error_dialog "FITS FILE: No value found for keyword: HIERARCH ESO OBS PROG ID" .skycat1
          return -code return
        }
        regsub -all {[(]} $progid "" progid
        regsub -all {[)]} $progid "" progid
        
        set alpha [image2 fits get "CRVAL1"]
        if {$alpha==""} {
          error_dialog "FITS FILE: No value found for keyword: CRVAL1" .skycat1
          return -code return
        }
        
        set delta [image2 fits get "CRVAL2"]
        if {$delta==""} {
          error_dialog "FITS FILE: No value found for keyword: CRVAL2" .skycat1
          return -code return
        }

        set coords [coord_format $alpha $delta]
        set alpha [lindex $coords 0]
        set delta [lindex $coords 1]
        
        set equinox [image2 fits get "EQUINOX"]
        if {$equinox==""} {
          error_dialog "FITS FILE: No value found for keyword: EQUINOX" .skycat1
          return -code return
        }
        
        set focu_temp [image2 fits get "HIERARCH ESO INS FOCU$quad TEMP"]
        if {$focu_temp==""} {
          error_dialog "FITS FILE: No value found for keyword: HIERARCH ESO INS FOCU$quad TEMP" .skycat1
          return -code return
        }
        
        set date_obs [image2 fits get "DATE-OBS"]
        if {$date_obs==""} {
          error_dialog "FITS FILE: No value found for keyword: DATE-OBS" .skycat1
          return -code return
        }
        
        set gui_ra [image2 fits get "HIERARCH ESO ADA GUID RA"]
        if {$gui_ra==""} {set gui_ra 0}
        
        set gui_dec  [image2 fits get "HIERARCH ESO ADA GUID DEC"]
        if {$gui_dec==""} {set gui_dec 0}

	set gui_ppos  [image2 fits get "HIERARCH ESO ADA ABSROT PPOS"]
        if {$gui_ppos==""} {set gui_ppos 0}

        set coords [coord_format $gui_ra $gui_dec]
        set gui_ra [lindex $coords 0]
        set gui_dec [lindex $coords 1]

        set pos_ang  [image2 fits get "HIERARCH ESO ADA POSANG"]
        if {$pos_ang==""} {
          error_dialog "FITS FILE: No value found for keyword: HIERARCH ESO ADA POSANG" .skycat1
          return -code return
        }
        
        set daytim  [image2 fits get "HIERARCH ESO PRO CCD MASK DAYTIM"]
        if {$daytim==""} {
          error_dialog "FITS FILE: No value found for keyword: HIERARCH ESO PRO CCD MASK DAYTIM" .skycat1
          return -code return
        }
        
        set temper  [image2 fits get "HIERARCH ESO PRO CCD MASK TEMP"]
        if {$temper==""} {
          error_dialog "FITS FILE: No value found for keyword: HIERARCH ESO PRO CCD MASK TEMP" .skycat1
          return -code return
        }

#        set tmp [open $name\_tmp a]
	#puts "DB> Opening $name\_tmp for writing"
        set tmp [open $name\_tmp w]
        puts $tmp "QueryResult"
        puts $tmp ""
        puts $tmp "# Config entry for original catalog server:"
        puts $tmp "serv_type: local"
        puts $tmp "long_name: $name"
        puts $tmp "short_name: [file tail $name]"
        puts $tmp "url: $name"
	puts $tmp "symbol: \{ALPHA_VIMOS DELTA_VIMOS FLAG\} \{circle white \{\} \{\} \{\} \{\$FLAG == \"S\"\}\} \{15 \{\}\} : \{ALPHA_VIMOS DELTA_VIMOS FLAG\} \{triangle blue \{\} \{\} \{\} \{\$FLAG == \"C\"\}\} \{15 \{\}\} : \{ALPHA_VIMOS DELTA_VIMOS FLAG\} \{diamond cyan \{\} \{\} \{\} \{\$FLAG == \"R\"\}\} \{20 \{\}\} : \{ALPHA_VIMOS DELTA_VIMOS FLAG\} \{cross yellow \{\} \{\} \{\} \{\$FLAG == \"F\"\}\} \{15 \{\}\} : \{ALPHA_VIMOS DELTA_VIMOS FLAG\} \{square green \{\} \{\} \{\} \{\$FLAG == \"A\"\}\} \{15 \{\}\}"
        puts $tmp "# Fits keywords"
        puts $tmp "#fits:INSTRUME \"$instrume\""
        puts $tmp "#fits:OCS.CON.QUAD \"$quad\""
        puts $tmp "#fits:OBS.PROG.ID \"$progid\""
        puts $tmp "#fits:TEL.TARG.ALPHA \"$alpha\""
        puts $tmp "#fits:TEL.TARG.DELTA \"$delta\""
        puts $tmp "#fits:TEL.TARG.EQUINOX \"$equinox\""
        puts $tmp "#fits:INS.FOCU$quad.TEMP \"$focu_temp\""
        puts $tmp "#fits:DATE-OBS \"$date_obs\""
        puts $tmp "#fits:TEL.GS1.ALPHA \"$gui_ra\"" 
        puts $tmp "#fits:TEL.GS1.DELTA \"$gui_dec\""  
	puts $tmp "#fits:TEL.GS1.PPOS \"$gui_ppos\""  
        puts $tmp "#fits:ADA.POSANG \"$pos_ang\""  
        puts $tmp "#fits:PRO.CCD.MASK.DAYTIM \"$daytim\""  
        puts $tmp "#fits:PRO.CCD.MASK.TEMP \"$temper\""  


################## MASK 2 CCD X MATRIX ##################

        set fitskey "HIERARCH ESO PRO CCD MASK X0"
        set val [image2 fits get $fitskey]
        if {$val==""} {
          error_dialog "FITS FILE: No value found for keyword: $fitskey" .skycat1
          return -code return
        }
        regsub -all " " $fitskey "." pafkey
        regsub "HIERARCH.ESO." $pafkey "" pafkey
        puts $tmp "#fits:$pafkey \"$val\""

        set fitskey "HIERARCH ESO PRO CCD MASK XX"
        set val [image2 fits get $fitskey]
        if {$val==""} {
          error_dialog "FITS FILE: No value found for keyword: $fitskey" .skycat1
          return -code return
        }
        regsub -all " " $fitskey "." pafkey
        regsub "HIERARCH.ESO." $pafkey "" pafkey
        puts $tmp "#fits:$pafkey \"$val\""

        set fitskey "HIERARCH ESO PRO CCD MASK XY"
        set val [image2 fits get $fitskey]
        if {$val==""} {
          error_dialog "FITS FILE: No value found for keyword: $fitskey" .skycat1
          return -code return
        }
        regsub -all " " $fitskey "." pafkey
        regsub "HIERARCH.ESO." $pafkey "" pafkey
        puts $tmp "#fits:$pafkey \"$val\""
        
        set fitskey "HIERARCH ESO PRO CCD MASK X_0_0"
        set val [image2 fits get $fitskey]
        if {$val==""} {
          error_dialog "FITS FILE: No value found for keyword: $fitskey" .skycat1
          return -code return
        }
        regsub -all " " $fitskey "." pafkey
        regsub "HIERARCH.ESO." $pafkey "" pafkey
        puts $tmp "#fits:$pafkey \"$val\""

        set fitskey "HIERARCH ESO PRO CCD MASK X_0_1"
        set val [image2 fits get $fitskey]
        if {$val==""} {
          error_dialog "FITS FILE: No value found for keyword: $fitskey" .skycat1
          return -code return
        }
        regsub -all " " $fitskey "." pafkey
        regsub "HIERARCH.ESO." $pafkey "" pafkey
        puts $tmp "#fits:$pafkey \"$val\""

        set fitskey "HIERARCH ESO PRO CCD MASK X_0_2"
        set val [image2 fits get $fitskey]
        if {$val==""} {
          error_dialog "FITS FILE: No value found for keyword: $fitskey" .skycat1
          return -code return
        }
        regsub -all " " $fitskey "." pafkey
        regsub "HIERARCH.ESO." $pafkey "" pafkey
        puts $tmp "#fits:$pafkey \"$val\""

        set fitskey "HIERARCH ESO PRO CCD MASK X_0_3"
        set val [image2 fits get $fitskey]
        if {$val==""} {
          error_dialog "FITS FILE: No value found for keyword: $fitskey" .skycat1
          return -code return
        }
        regsub -all " " $fitskey "." pafkey
        regsub "HIERARCH.ESO." $pafkey "" pafkey
        puts $tmp "#fits:$pafkey \"$val\""

        set fitskey "HIERARCH ESO PRO CCD MASK X_1_0"
        set val [image2 fits get $fitskey]
        if {$val==""} {
          error_dialog "FITS FILE: No value found for keyword: $fitskey" .skycat1
          return -code return
        }
        regsub -all " " $fitskey "." pafkey
        regsub "HIERARCH.ESO." $pafkey "" pafkey
        puts $tmp "#fits:$pafkey \"$val\""

        set fitskey "HIERARCH ESO PRO CCD MASK X_1_1"
        set val [image2 fits get $fitskey]
        if {$val==""} {
          error_dialog "FITS FILE: No value found for keyword: $fitskey" .skycat1
          return -code return
        }
        regsub -all " " $fitskey "." pafkey
        regsub "HIERARCH.ESO." $pafkey "" pafkey
        puts $tmp "#fits:$pafkey \"$val\""

        set fitskey "HIERARCH ESO PRO CCD MASK X_1_2"
        set val [image2 fits get $fitskey]
        if {$val==""} {
          error_dialog "FITS FILE: No value found for keyword: $fitskey" .skycat1
          return -code return
        }
        regsub -all " " $fitskey "." pafkey
        regsub "HIERARCH.ESO." $pafkey "" pafkey
        puts $tmp "#fits:$pafkey \"$val\""

        set fitskey "HIERARCH ESO PRO CCD MASK X_1_3"
        set val [image2 fits get $fitskey]
        if {$val==""} {
          error_dialog "FITS FILE: No value found for keyword: $fitskey" .skycat1
          return -code return
        }
        regsub -all " " $fitskey "." pafkey
        regsub "HIERARCH.ESO." $pafkey "" pafkey
        puts $tmp "#fits:$pafkey \"$val\""

        set fitskey "HIERARCH ESO PRO CCD MASK X_2_0"
        set val [image2 fits get $fitskey]
        if {$val==""} {
          error_dialog "FITS FILE: No value found for keyword: $fitskey" .skycat1
          return -code return
        }
        regsub -all " " $fitskey "." pafkey
        regsub "HIERARCH.ESO." $pafkey "" pafkey
        puts $tmp "#fits:$pafkey \"$val\""

        set fitskey "HIERARCH ESO PRO CCD MASK X_2_1"
        set val [image2 fits get $fitskey]
        if {$val==""} {
          error_dialog "FITS FILE: No value found for keyword: $fitskey" .skycat1
          return -code return
        }
        regsub -all " " $fitskey "." pafkey
        regsub "HIERARCH.ESO." $pafkey "" pafkey
        puts $tmp "#fits:$pafkey \"$val\""

        set fitskey "HIERARCH ESO PRO CCD MASK X_2_2"
        set val [image2 fits get $fitskey]
        if {$val==""} {
          error_dialog "FITS FILE: No value found for keyword: $fitskey" .skycat1
          return -code return
        }
        regsub -all " " $fitskey "." pafkey
        regsub "HIERARCH.ESO." $pafkey "" pafkey
        puts $tmp "#fits:$pafkey \"$val\""

        set fitskey "HIERARCH ESO PRO CCD MASK X_2_3"
        set val [image2 fits get $fitskey]
        if {$val==""} {
          error_dialog "FITS FILE: No value found for keyword: $fitskey" .skycat1
          return -code return
        }
        regsub -all " " $fitskey "." pafkey
        regsub "HIERARCH.ESO." $pafkey "" pafkey
        puts $tmp "#fits:$pafkey \"$val\""

        set fitskey "HIERARCH ESO PRO CCD MASK X_3_0"
        set val [image2 fits get $fitskey]
        if {$val==""} {
          error_dialog "FITS FILE: No value found for keyword: $fitskey" .skycat1
          return -code return
        }
        regsub -all " " $fitskey "." pafkey
        regsub "HIERARCH.ESO." $pafkey "" pafkey
        puts $tmp "#fits:$pafkey \"$val\""

        set fitskey "HIERARCH ESO PRO CCD MASK X_3_1"
        set val [image2 fits get $fitskey]
        if {$val==""} {
          error_dialog "FITS FILE: No value found for keyword: $fitskey" .skycat1
          return -code return
        }
        regsub -all " " $fitskey "." pafkey
        regsub "HIERARCH.ESO." $pafkey "" pafkey
        puts $tmp "#fits:$pafkey \"$val\""

        set fitskey "HIERARCH ESO PRO CCD MASK X_3_2"
        set val [image2 fits get $fitskey]
        if {$val==""} {
          error_dialog "FITS FILE: No value found for keyword: $fitskey" .skycat1
          return -code return
        }
        regsub -all " " $fitskey "." pafkey
        regsub "HIERARCH.ESO." $pafkey "" pafkey
        puts $tmp "#fits:$pafkey \"$val\""

        set fitskey "HIERARCH ESO PRO CCD MASK X_3_3"
        set val [image2 fits get $fitskey]
        if {$val==""} {
          error_dialog "FITS FILE: No value found for keyword: $fitskey" .skycat1
          return -code return
        }
        regsub -all " " $fitskey "." pafkey
        regsub "HIERARCH.ESO." $pafkey "" pafkey
        puts $tmp "#fits:$pafkey \"$val\""


################## MASK 2 CCD Y MATRIX ##################

        set fitskey "HIERARCH ESO PRO CCD MASK Y0"
        set val [image2 fits get $fitskey]
        if {$val==""} {
          error_dialog "FITS FILE: No value found for keyword: $fitskey" .skycat1
          return -code return
        }
        regsub -all " " $fitskey "." pafkey
        regsub "HIERARCH.ESO." $pafkey "" pafkey
        puts $tmp "#fits:$pafkey \"$val\""

        set fitskey "HIERARCH ESO PRO CCD MASK YY"
        set val [image2 fits get $fitskey]
        if {$val==""} {
          error_dialog "FITS FILE: No value found for keyword: $fitskey" .skycat1
          return -code return
        }
        regsub -all " " $fitskey "." pafkey
        regsub "HIERARCH.ESO." $pafkey "" pafkey
        puts $tmp "#fits:$pafkey \"$val\""

        set fitskey "HIERARCH ESO PRO CCD MASK YX"
        set val [image2 fits get $fitskey]
        if {$val==""} {
          error_dialog "FITS FILE: No value found for keyword: $fitskey" .skycat1
          return -code return
        }
        regsub -all " " $fitskey "." pafkey
        regsub "HIERARCH.ESO." $pafkey "" pafkey
        puts $tmp "#fits:$pafkey \"$val\""
        
        set fitskey "HIERARCH ESO PRO CCD MASK Y_0_0"
        set val [image2 fits get $fitskey]
        if {$val==""} {
          error_dialog "FITS FILE: No value found for keyword: $fitskey" .skycat1
          return -code return
        }
        regsub -all " " $fitskey "." pafkey
        regsub "HIERARCH.ESO." $pafkey "" pafkey
        puts $tmp "#fits:$pafkey \"$val\""

        set fitskey "HIERARCH ESO PRO CCD MASK Y_0_1"
        set val [image2 fits get $fitskey]
        if {$val==""} {
          error_dialog "FITS FILE: No value found for keyword: $fitskey" .skycat1
          return -code return
        }
        regsub -all " " $fitskey "." pafkey
        regsub "HIERARCH.ESO." $pafkey "" pafkey
        puts $tmp "#fits:$pafkey \"$val\""

        set fitskey "HIERARCH ESO PRO CCD MASK Y_0_2"
        set val [image2 fits get $fitskey]
        if {$val==""} {
          error_dialog "FITS FILE: No value found for keyword: $fitskey" .skycat1
          return -code return
        }
        regsub -all " " $fitskey "." pafkey
        regsub "HIERARCH.ESO." $pafkey "" pafkey
        puts $tmp "#fits:$pafkey \"$val\""

        set fitskey "HIERARCH ESO PRO CCD MASK Y_0_3"
        set val [image2 fits get $fitskey]
        if {$val==""} {
          error_dialog "FITS FILE: No value found for keyword: $fitskey" .skycat1
          return -code return
        }
        regsub -all " " $fitskey "." pafkey
        regsub "HIERARCH.ESO." $pafkey "" pafkey
        puts $tmp "#fits:$pafkey \"$val\""

        set fitskey "HIERARCH ESO PRO CCD MASK Y_1_0"
        set val [image2 fits get $fitskey]
        if {$val==""} {
          error_dialog "FITS FILE: No value found for keyword: $fitskey" .skycat1
          return -code return
        }
        regsub -all " " $fitskey "." pafkey
        regsub "HIERARCH.ESO." $pafkey "" pafkey
        puts $tmp "#fits:$pafkey \"$val\""

        set fitskey "HIERARCH ESO PRO CCD MASK Y_1_1"
        set val [image2 fits get $fitskey]
        if {$val==""} {
          error_dialog "FITS FILE: No value found for keyword: $fitskey" .skycat1
          return -code return
        }
        regsub -all " " $fitskey "." pafkey
        regsub "HIERARCH.ESO." $pafkey "" pafkey
        puts $tmp "#fits:$pafkey \"$val\""

        set fitskey "HIERARCH ESO PRO CCD MASK Y_1_2"
        set val [image2 fits get $fitskey]
        if {$val==""} {
          error_dialog "FITS FILE: No value found for keyword: $fitskey" .skycat1
          return -code return
        }
        regsub -all " " $fitskey "." pafkey
        regsub "HIERARCH.ESO." $pafkey "" pafkey
        puts $tmp "#fits:$pafkey \"$val\""

        set fitskey "HIERARCH ESO PRO CCD MASK Y_1_3"
        set val [image2 fits get $fitskey]
        if {$val==""} {
          error_dialog "FITS FILE: No value found for keyword: $fitskey" .skycat1
          return -code return
        }
        regsub -all " " $fitskey "." pafkey
        regsub "HIERARCH.ESO." $pafkey "" pafkey
        puts $tmp "#fits:$pafkey \"$val\""

        set fitskey "HIERARCH ESO PRO CCD MASK Y_2_0"
        set val [image2 fits get $fitskey]
        if {$val==""} {
          error_dialog "FITS FILE: No value found for keyword: $fitskey" .skycat1
          return -code return
        }
        regsub -all " " $fitskey "." pafkey
        regsub "HIERARCH.ESO." $pafkey "" pafkey
        puts $tmp "#fits:$pafkey \"$val\""

        set fitskey "HIERARCH ESO PRO CCD MASK Y_2_1"
        set val [image2 fits get $fitskey]
        if {$val==""} {
          error_dialog "FITS FILE: No value found for keyword: $fitskey" .skycat1
          return -code return
        }
        regsub -all " " $fitskey "." pafkey
        regsub "HIERARCH.ESO." $pafkey "" pafkey
        puts $tmp "#fits:$pafkey \"$val\""

        set fitskey "HIERARCH ESO PRO CCD MASK Y_2_2"
        set val [image2 fits get $fitskey]
        if {$val==""} {
          error_dialog "FITS FILE: No value found for keyword: $fitskey" .skycat1
          return -code return
        }
        regsub -all " " $fitskey "." pafkey
        regsub "HIERARCH.ESO." $pafkey "" pafkey
        puts $tmp "#fits:$pafkey \"$val\""

        set fitskey "HIERARCH ESO PRO CCD MASK Y_2_3"
        set val [image2 fits get $fitskey]
        if {$val==""} {
          error_dialog "FITS FILE: No value found for keyword: $fitskey" .skycat1
          return -code return
        }
        regsub -all " " $fitskey "." pafkey
        regsub "HIERARCH.ESO." $pafkey "" pafkey
        puts $tmp "#fits:$pafkey \"$val\""

        set fitskey "HIERARCH ESO PRO CCD MASK Y_3_0"
        set val [image2 fits get $fitskey]
        if {$val==""} {
          error_dialog "FITS FILE: No value found for keyword: $fitskey" .skycat1
          return -code return
        }
        regsub -all " " $fitskey "." pafkey
        regsub "HIERARCH.ESO." $pafkey "" pafkey
        puts $tmp "#fits:$pafkey \"$val\""

        set fitskey "HIERARCH ESO PRO CCD MASK Y_3_1"
        set val [image2 fits get $fitskey]
        if {$val==""} {
          error_dialog "FITS FILE: No value found for keyword: $fitskey" .skycat1
          return -code return
        }
        regsub -all " " $fitskey "." pafkey
        regsub "HIERARCH.ESO." $pafkey "" pafkey
        puts $tmp "#fits:$pafkey \"$val\""

        set fitskey "HIERARCH ESO PRO CCD MASK Y_3_2"
        set val [image2 fits get $fitskey]
        if {$val==""} {
          error_dialog "FITS FILE: No value found for keyword: $fitskey" .skycat1
          return -code return
        }
        regsub -all " " $fitskey "." pafkey
        regsub "HIERARCH.ESO." $pafkey "" pafkey
        puts $tmp "#fits:$pafkey \"$val\""

        set fitskey "HIERARCH ESO PRO CCD MASK Y_3_3"
        set val [image2 fits get $fitskey]
        if {$val==""} {
          error_dialog "FITS FILE: No value found for keyword: $fitskey" .skycat1
          return -code return
        }
        regsub -all " " $fitskey "." pafkey
        regsub "HIERARCH.ESO." $pafkey "" pafkey
        puts $tmp "#fits:$pafkey \"$val\""

        puts $tmp "# End fits keywords"
        puts $tmp "# Curved slits"
        puts $tmp "# End curved slits"
        puts $tmp "# End config entry"
        ::close $tmp

	# AAG - Input files are supposed to be ascii only
	# If skycat, add the following lines
        #set n [exec grep -n "End config entry" $name | cut -d: -f1]
        #set n [expr $n + 1]
	catch {set heading [exec grep ID $name]}
	if {$heading != ""} {
	    set buf [exec grep ID $name]
	} else {
	    set buf [exec grep id $name]
	}
	set buf [string toupper $buf]
	set Ncol [llength $buf]
	set val_line [lindex $buf 0]
        for {set n 1} {$n < $Ncol} {incr n 1} {
	    set val [lindex $buf $n]
	    set val_line $val_line\t$val
	}
	exec echo $val_line >> $name\_tmp
	exec echo "---" >> $name\_tmp
	set n 2
        exec tail +$n $name > $name\_tmp_1
	set fId_1 [::open $name\_tmp_1 r ]
	set fId_2 [::open $name\_tmp_2 w ]
	
	while {[gets $fId_1 line] >= 0} {
	    set Ncol [llength $line]
	    set val_line [lindex $line 0]
	    for {set n 1} {$n < $Ncol} {incr n 1} {
		set val [lindex $line $n]
		set val_line $val_line\t$val
	    }
	    puts $fId_2 $val_line
	}
	::close $fId_1
	::close $fId_2
	exec cat $name\_tmp_2 >> $name\_tmp
	catch {file delete $name\_tmp_1}
	catch {file delete $name\_tmp_2}
	# AAG- Keep original ascii file and 
	# work with temporal file
#        catch {file delete $name}
#        file rename -force $name\.tmp $name
        sleep 3
     }


######################### New method for vmmps #######################
    public method export {} {

      global catname
      global quad
      global mps_window


      set quad [exec grep OCS.CON.QUAD $catname | cut -f2 -d\"]

      set f_ [::open $catname r ]
      set f2_ [::open $catname\_2 w ]	
      set indexRAD -1
      set indexFLAG -1
      set nRef 0 
      set f_header 0
      while {[gets $f_ line] >= 0} {
	if {!$f_header} {
		set line1 [string toupper $line]
       	 	if {[string match *X_RADIUS* $line1]} {
	    		set indexRAD [lsearch -exact $line X_RADIUS]
	    		set indexFLAG [lsearch -exact $line FLAG]
	 	} 
                if {[string match *---* $line]} {
                    set f_header 1
                }
	} else {
	    set val_rad [lindex $line $indexRAD]
	    set val_flag [lindex $line $indexFLAG]
	    if {$val_rad > 45} {
		#puts "DB> Found X_RADIUS: $val_rad > 45 -> Forbidden object"
		set line [lreplace $line $indexFLAG $indexFLAG "F"]
		set res [regsub -all " " $line \t line]
	    }
	    if {$val_flag=="R"} {
		set nRef [expr $nRef + 1]
	    }
	}
	puts $f2_ $line
      }	
      ::close $f_
      ::close $f2_

      if {$nRef == 0} {
	 error_dialog "At least one Reference Star is Mandatory. Please select one" .skycat1
         catch {file delete $catname\_2}
	 return
      }

      catch {file rename -force $catname\_2 $catname}

      $this.results mpsprint $catname

      set command  "namespace inscope ::cat::vmAstroCat {::$mps_window search}"
      eval $command

      utilReUseWidget cat::vmmps_spoc_gui .skycat1.spoc

      set dat [open $catname\_arc w]
      catch {puts $dat [exec grep #arc $catname | cut -f2- -d:]}
      ::close $dat

      exec vmmps_dim [file rootname $catname].dat_tmp $catname\_arc [file rootname $catname].dat
      catch {file delete [file rootname $catname].dat_tmp}
      catch {file delete $catname\_arc}

    }


    # disable the search button, if the arg is 1, otherwise enable it

    public method disable_search {{arg 1}} {
	if {$arg} {
	    set search_state_ disabled
	} else {
	     set search_state_ normal
	}
    }

    
    # if the catalog has a "help" URL, try to display it in netscape using the
    # netscape remote interface.

    public method help {} {
	set url [$w_.cat help]
	if {"$url" != ""} {
	    send_to_netscape $url
	} else {
	    info_dialog "Sorry, no help is available for this catalog"
	}
    }


    # add a progress bar to display the progress of data transfers

    protected method add_progress_bar {} {
	# add a progress bar at the botton
	pack [ProgressBar $w_.progress] \
	    -side top -fill x
    }
    

    # do the dialog window layout
    
    protected method layout_dialog {} {
	open_catalog
	add_menubar
	add_search_options
	get_name_servers
	if {$iscat_} {
	    add_result_table
	}
        add_dialog_buttons
	add_progress_bar
    }


    # add a short help window and set the help texts
    
    protected method make_short_help {} {
	TopLevelWidget::make_short_help

	add_short_help $results_ \
	    {Query results: {bitmap b1} = select object, \
	       double click {bitmap b1} = label object, \
	                    {bitmap dragb2} = scroll list}

	if {$iscat_} {
	    add_short_help $w_.search \
		{{bitmap b1} = start catalog search}

	    add_short_help $w_.preview \
		{{bitmap b1} = view the preview image or plot data for the selected object}

	    add_short_help $w_.more \
		{{bitmap b1} = display more information for the selected object}

	} else {
	    add_short_help $w_.search \
		{{bitmap b1} = fetch the image from the image server}
	}

	add_short_help $w_.stop \
	    {{bitmap b1} = interrupt the current catalog operation}

	if {[winfo exists $w_.help]} {
	    add_short_help $w_.help \
		{{bitmap b1} = Display help information for catalog in netscape}
	}

	add_short_help $w_.close {{bitmap b1} = close this window}

	add_short_help $w_.progress {Progress bar: displays status of work in progress}

    }




# -----------------------------------------------------------------------------
#
#                          H a n d h o l d e r
#
#   This procedure puts up the 'handholding' basic sequence guide.
#

	  public proc Handholder { } {

	      global flag_use         ;# 1 means use of this sequence panel    
	      set flag_use 1

	      global AsciiFile          ;# Currently Ascii catalog
	      global AsciiFile_conv     ;# Currently Skycat catalog
	      global StarFile           ;# Current Reference catalog
	      global FitsFile           ;# Current displayedFits File 
	      global config_font_1      ;# Font used in dialogues
	      global TickDetails        ;# Details of ticks in the basic sequence dialogue

	      set config_font_1 {-Adobe-Times-Medium-R-Normal-*-180-*}
	      set config_font_2 {-Adobe-Helvetica-Medium-R-Normal-*-120-*}
	      set FitsFile ""
	      set AsciiFile ""
	      set AsciiFile_conv ""
	      set StarFile ""


	      #  TickDetails is used to record the state of the tick marks following
	      #  each button, and forms a linked list that can be used to clear all
	      #  the ticks from a named entry down. The 'displayed' entry is supposed
	      #  to handle cases where an item is not used for a specific instrument,
	      #  but the entry is still needed to maintain the linked list.
	      
	      toplevel .handhold
	      wm title .handhold "VIMOS: Cross Correlation sequence"
	      
	      set level 1

	      #  Open ASCII Catalog

	      frame .handhold.opena -relief raised -borderwidth 1
	      button .handhold.opena.button -command [code cat::vmAstroCat::OpenAsciiProc] -width 40 \
		  -text "Select Contributed catalog (ascii)"
	      label .handhold.opena.tick -bitmap tick -foreground red
	      label .handhold.opena.filename -textvariable AsciiFile -width 70 \
		  -font $config_font_1
	      pack .handhold.opena.button -side left -fill both
	      pack .handhold.opena.tick -side left -fill both
	      pack .handhold.opena.filename -side left -fill both
	      raise .handhold.opena.button
	      pack .handhold.opena
	      
	      set TickDetails(OpenAscii,widget) .handhold.opena.tick
	      set TickDetails(OpenAscii,displayed) 1
	      set TickDetails(OpenAscii,next) OpenStar
	      set TickDetails(OpenAscii,level) $level
	      incr level


	      #  Open Star Catalogue

	      frame .handhold.opens -relief raised -borderwidth 1
	      button .handhold.opens.button -command [code cat::vmAstroCat::OpenStarProc] -width 40 \
		  -text "Select VIMOS Pre-image catalog (ascii)"
	      label .handhold.opens.tick -bitmap tick -foreground red
	      label .handhold.opens.filename -textvariable StarFile -width 70 \
		  -font $config_font_1
	      pack .handhold.opens.button -side left -fill both
	      pack .handhold.opens.tick -side left -fill both
	      pack .handhold.opens.filename -side left -fill both
	      raise .handhold.opens.button
	      pack .handhold.opens
	      
	      set TickDetails(OpenStar,widget) .handhold.opens.tick
	      set TickDetails(OpenStar,displayed) 1
	      set TickDetails(OpenStar,next) OpenFits
	      set TickDetails(OpenStar,level) $level
	      incr level

	      #  Open Image, Fits File
	      
	      frame .handhold.openf -relief raised -borderwidth 1
	      button .handhold.openf.button -command [code cat::vmAstroCat::OpenFitsProc] -width 40 \
		  -text "Select VIMOS Pre-image File (fits)"
	      label .handhold.openf.tick -bitmap tick -foreground red
	      label .handhold.openf.filename -textvariable FitsFile -width 70 \
		  -font $config_font_1
	      pack .handhold.openf.button -side left -fill both
	      pack .handhold.openf.tick -side left -fill both
	      pack .handhold.openf.filename -side left -fill both
	      raise .handhold.openf.button
	      pack .handhold.openf
	      
	      set TickDetails(OpenFits,widget) .handhold.openf.tick
	      set TickDetails(OpenFits,displayed) 1
	      set TickDetails(OpenFits,next) "" 
	      set TickDetails(OpenFits,level) $level
	      incr level
	    
	      # Cross 
	
	      frame .handhold.cross -relief raised -borderwidth 1
	      button .handhold.cross.button -command [code cat::vmAstroCat::DoCross] -width 40 \
		  -text "Cross Correlate"
	      label .handhold.cross.filename -textvariable cross_msg -width 71 \
		  -font $config_font_1
	      pack .handhold.cross.button -side left -fill both
	      pack .handhold.cross.filename -side left -fill both
	      raise .handhold.cross.button
	      pack .handhold.cross
	      
	      incr level
	     


	      wm geometry .handhold +0+0
	      
	      #  Don't allow the user to close the window
	      
	      #wm protocol .handhold WM_DELETE_WINDOW NullAction
	      
	      #  This works to clear all the ticks - if there were other entries at
	      #  the same level as MagFilter we'd need some other ploy, as this
	      #  would leave those other entries ticked.
	      
	      ClearTicks OpenFits
	      ClearTicks OpenAscii
	      ClearTicks OpenStar
	      
	  }


#------------------------------------------------------------------------------
#
#                       O p e n  A s c i i
#
#   Procedure called as a result of the "Open..." selection from the
#   File menu. Puts up a file selection dialog to get the file name
#   and then call OpenAsciiOk to read it in it when the OK button is
#   selected.


public proc OpenAsciiProc {} {

    global AsciiFile         ;# Currently Ascii catalog
    global AsciiFile_conv    ;# Currently Skycat catalog
    global StarFile
    global FitsFile         ;# Currently displayed Image

    set ext "*"

    set catname ""
    set catname_in "" 
    set AsciiFile_conv ""

    set AsciiFile ""
    set FitsFile ""
    set StarFile ""
    ClearTicks OpenFits
    ClearTicks OpenAscii
    ClearTicks OpenStar

    info_dialog "Select the Contributed catalog" .skycat1
    set catname_in [filename_dialog [pwd] *.$ext .skycat1]

    if {"$catname_in" != ""} {

	if {[file isfile $catname_in]} {
	    #puts "DB> Input Ascii Catalog: $catname_in"
	    set catname [ascii2skycat $catname_in]
	    if {$catname == 0} {
		return
	    }
	    #puts "DB> Output Skycat Catalog: $catname"
	    if {[catch {$astrocat_ check $catname} msg]} {
		error_dialog $msg .skycat1
	    } 
	} else {
	    error_dialog "There is no file named '$catname_in'" .skycat1
	    return
	}

    set AsciiFile $catname_in
    set AsciiFile_conv $catname
    SetTick OpenAscii

    }
}

public proc OpenStarProc {} {

    global StarFile
    global FitsFile         ;# Currently displayed Image
    global AsciiFile_conv   ;# Currentle Skycat catalog
    set id "namespace inscope ::skycat::SkyCat .skycat1.image"
    set classname "::skycat::vmSkySearch"
    set debug 0
    set w ""
    set StarFile ""
    set FitsFile ""

    ClearTicks OpenStar
    ClearTicks OpenFits

    if {$AsciiFile_conv != ""} {
	set StarFile [select_catalog $AsciiFile_conv local $id $classname \
			  [is_tcs_catalog $AsciiFile_conv] $debug $w ]
	if {$StarFile != -1} {
	    SetTick OpenStar
	}
    } else {
        error_dialog "Load Contributed catalog first" .skycat1
    }
}

public proc OpenFitsProc {} {

    global FitsFile         ;# Currently displayed Image
    global StarFile         ;# Currently displayed Image
    global AsciiFile_conv   ;# Currently Skycat catalog 
    global imaname

    set vimos ""
    set imaname ""

    set FitsFile ""
    ClearTicks OpenFits

    if {$AsciiFile_conv == ""} {
        error_dialog "Load Contributed catalog first" .skycat1
	return
    }
    if {$StarFile == "" || $StarFile == -1} {
        error_dialog "Load Reference catalog first" .skycat1
	return
    }

    catch {set vimos [exec grep VIMOS $AsciiFile_conv]}
    set instrume ""

#    set instrume [image2 fits get "INSTRUME"]
#    set imaname [image2 cget -file]
     if {$instrume != "VIMOS"} {
	info_dialog "Select the VIMOS pre-image" .skycat1
	.skycat1.image open
	set skycat "namespace inscope ::skycat::SkyCat .skycat1.image"
	set image_ [$skycat get_image]
	set imaname [$image_ cget -file]
	set instrume [image2 fits get "INSTRUME"]
	if {$instrume != "VIMOS"} {
		set imaname ""
        	error_dialog " Fits file VIMOS image ..." .skycat1
		return
	}
      }

     if {$imaname == ""} {
          set skycat "namespace inscope ::skycat::SkyCat .skycat1.image"
	  set image_ [$skycat get_image]
	  set imaname [image2_ cget -file]
     }

     if {$imaname != ""} {
       #puts "DB> Loading $imaname"
       set FitsFile $imaname
       SetTick OpenFits
       }
}

public proc DoCross {} {
    global tot_first tot_last

    global FitsFile         ;# Currently displayed Image
    global StarFile         ;# Currently displayed Image
    global AsciiFile_conv   ;# Currently Skycat catalog

    if {$AsciiFile_conv == ""} {
        error_dialog "Load Contributed catalog first" .skycat1
        return
    }
    if {$StarFile == "" || $StarFile == -1} {
        error_dialog "Load Reference catalog first" .skycat1
        return
    }
    if {$FitsFile == "" || $FitsFile == -1} {
        error_dialog "Load VIMOS pre-image first" .skycat1
        return
    }


       set tot_first 0
       set tot_last 0
       utilReUseWidget cat::vmmps_cross_gui .skycat1.cross
}


# -----------------------------------------------------------------------------
#
#                             E x i t  A p p
#
#   This procedure is tied to the Exit option in the file menu. It closes
#   down the program, first checking if any modified file is to be saved.

public proc ExitApp {} {
      global rootbase catname
      global catname_for_newslit

      #puts "DB> Removing temporal files in $rootbase"
      catch {eval "file delete [glob -nocomplain [file dirname $rootbase]/*.adp_cat]"}
      #puts "DB> [file dirname $rootbase]/*.adp_cat removed"
      #catch {eval "file delete [glob -nocomplain [file dirname $rootbase]/*.BAK]"}
      #puts "DB> [file dirname $rootbase]/*.BAK removed"
      #catch {eval "file delete [glob -nocomplain [file dirname $rootbase]/*.log]"}
      #puts "DB> [file dirname $rootbase]/*.log removed"
      #catch {eval "file delete [glob -nocomplain [file dirname $rootbase]/*.dat]"}
      #puts "DB> [file dirname $rootbase]/*.dat removed"      
      #catch {eval "file delete [glob -nocomplain [file dirname $rootbase]/*.tmp]"}
      #puts "DB> [file dirname $rootbase]/*.tmp removed"

      catch {eval "file delete [file rootname $catname_for_newslit].dat"}
      catch {eval "file delete [file rootname $catname_for_newslit].log"}
      catch {eval "file delete $catname_for_newslit.BAK"}
      #catch {eval "file delete $catname_for_newslit"}           


        destroy .
}


# -----------------------------------------------------------------------------
#
#                          C l e a r  T i c k s
#
#  This routine clears the tick from one of the entries in the basic sequence
#  dialogue and for all entries at a lower level. It is passed one of the
#  names used for the entries in the TickDetails global array - see
#  Handholder{} for more details. It clears the entry specified, and then
#  any other entries at a higher level. (So it doesn't clear other entries
#  at the same level as the named entry.)
#
#  This was a lot more straightforward until it was realised that in some
#  cases clearing an entry in the basic sequence set up by Handholder{} does
#  not always imply that all subsequent entries should be cleared. For
#  example, for FLAMES, selecting the guide probe and selecting the fibre
#  combination are independent and changing one does not invalidate the other.
#  These are really at the same 'level' in the sequence, which is why the
#  'level' concept was added to this code.

public proc ClearTicks { name } {

    #  GLobal variables used:

    global TickDetails      ;# Details of ticks in the basic sequence dialogue

    #  Allow for the possibility that we are being run before Handholder{}
    #  has set up the TickDetails array.

    if { [info exists TickDetails] } {

        set level $TickDetails($name,level)

        #  Clear the tick for this entry.

        ClearOneTick $name

        #  Work through all the entries further on in the sequence,
        #  clearing the ticks for any at a higher level.

        while { $TickDetails($name,next) != "" } {
           set NewName $TickDetails($name,next)
           set name $NewName
           if { $TickDetails($name,level) > $level } { ClearOneTick $name }
        }
    }
}

# -----------------------------------------------------------------------------
#
#                         C l e a r  O n e  T i c k
#
#  This routine clears the tick from one of the entries in the basic sequence,
#  by setting the colour for the tick to be the same colour as its background.
#  It is passed one of the names used for the entries in the TickDetails
#  global array - see Handholder{} for more details.

public proc ClearOneTick { name } {

    #  GLobal variables used:

    global TickDetails      ;# Details of ticks in the basic sequence dialogue

    if { $TickDetails($name,displayed) } {
        set widget $TickDetails($name,widget)
        set bgcolour [$widget cget -background]
        $widget configure -foreground $bgcolour
    }
    set TickDetails($name,ticked) 0
}    

       # -----------------------------------------------------------------------------
#
#                            S e t  T i c k
#
#  This routine ticks one of the entries in the basic sequence dialogue.
#  It should be called whenever the action associated with the entry has
#  taken place (whether initiated from the dialogue or not). It ticks the
#  entry itself and then clears all the ticks from subsequent entries - on
#  the assumption that a change earlier on in the sequence invalidates
#  any later actions. It is passed one of the names used for the entries
#  in the TickDetails global array - see Handholder{} for more details.

proc SetTick { name } {

    #  GLobal variables used:

    global TickDetails      ;# Details of ticks in the basic sequence dialogue

    #  Allow for the possibility that we are being run before Handholder{}
    #  has set up the TickDetails array.

    if { [info exists TickDetails] } {

        #  If the entry is being displayed, enable the tick by changing
        #  its colour to red.

        if { $TickDetails($name,displayed) } {
            set widget $TickDetails($name,widget)
            $widget configure -foreground red
        }
        set TickDetails($name,ticked) 1

        #  If there are entries further on in the sequence, clear their ticks.
        #  Note that we clear everything at a level greater than that of the
        #  entry whose tick we just set.

        set level $TickDetails($name,level)

        while { $TickDetails($name,next) != "" } {
           set newName $TickDetails($name,next)
           set name $newName
           if { $TickDetails($name,level) > $level } { ClearOneTick $name }
        }
    }
}




    ################### Modified for vmmps ###############################
    # This proc is called when the named catalog selected from the menu.
    #
    # serv_type is the type of the catalog: one of: catalog, archive, 
    # local, imagesvr, etc...
    #
    # id is an optional unique id to be associated with a new catalog widget.
    #
    # classname is the name of the vmAstroCat subclass to use to create new
    # catalog widgets (defaults to "vmAstroCat").
    #
    # tcs_flag can be set to true, if you know it is a TCS catalog.
    #
    # $debug is a flag set from the command line arg.
    #
    # $w should be the top level window of the caller (optional).

    public proc select_catalog {name serv_type id classname {tcs_flag 0} {debug 0} {w ""}} {

        global convert catname vimoscat
	global flag_use
        set convert "no"
        

        set number_vimos ""
        catch {set number_vimos [exec grep -i ID $name]}
        if {$number_vimos == ""} {
            return -code error "Column ID is missing"
        }

        set alpha_delta_vimos ""
	#AAG - It should check upper & lower case!
        catch {set alpha_delta_vimos [exec grep -i ALPHA_VIMOS $name | grep -i DELTA_VIMOS]}
        if {$alpha_delta_vimos == ""} {

           set x_y_image ""
	   #AAG - CHECK if wether we are in cross or spoc 
           catch {set x_y_image [exec grep -i X_IMAGE $name | grep -i Y_IMAGE]}
           if {$x_y_image == ""} {
              set alpha_delta ""
              catch {set alpha_delta [exec -i grep ALPHA_SKY $name | grep -i DELTA_SKY]}
              set x_y_world ""
              catch {set x_y_world [exec grep -i RA $name | grep -i DEC]}
              if {$x_y_world == "" && $alpha_delta == ""} {
                 return -code error "Columns X_IMAGE,Y_IMAGE or ALPHA_SKY,DELTA_SKY or RA,DEC are missing"
              }
              set catname $name        
              info_dialog "Select the VIMOS Pre-image catalog" .skycat1
              set vimoscat_in [filename_dialog [pwd] *.* .skycat1]
	      set vimoscat [ascii2txt $vimoscat_in]
	      if {$vimoscat != -1} {
              	#utilReUseWidget cat::vmmps_cross_gui .skycat1.cross
	       	if {$flag_use} {
		   return $vimoscat_in
	       	} else { 
		   return
	       	}
	      } else {
	           return -1
	      }
           }

           set convert "yes"
	   #puts "DB> Converting $name to skycat format"
           symbol $name
	   # AAG - Work with temporal file
	   set name $name\_tmp
           $astrocat_ entry remove $name
           cat::CatalogInfo::save {} $w 0
           $astrocat_ reload
        }
        new_catalog $name $id $classname $debug $tcs_flag "catalog" $w
 
    }
    
    # set the name server to use to resolve object names
    
    public method set_namesvr {name} {
	$searchopts_ config -namesvr $name

    }


    # interrupt the current search 

    public method interrupt {} {
	$w_.batch interrupt
	$searchopts_ interrupt
	set_feedback off
	catch {$results_ config -title "Search Results"}
	set_state normal
    }

    
    # set/reset widget states while busy 

    public method set_state {state} {
	set state_ $state
	if {"$state" == "normal"} {
	    catch {blt::busy release $w_.options}
	    catch {focus -lastfor $w_.options}
	    $w_.search config -state $search_state_
	    $w_.stop config -state disabled
	} else {
	    catch {focus .}
	    catch {blt::busy hold $w_.options}
	    $w_.search config -state disabled
	    catch {
		$w_.more config -state disabled
		$w_.preview config -state disabled
	    }
	    $w_.stop config -state normal
	}
	update idletasks
	$w_.progress reset
    }

    
    # this method is called by the fileevent handler during the data transfer 
    # from the HTTP C++ class to give the user feedback about how much there is 
    # left to copy, etc...
    # Read a line from the feedback pipe, which should contain text to be displayed
    # in the progress widget.
    #
    # When we are in busy (disabled, waiting) mode, we also interpret some special
    # messages, such as: "total length: n bytes" to update the progress bar.

    public method feedback {} {
	set text [gets $rfd_]
	if {"$state_" != "normal"} {
	    if {[scan $text {total length: %d bytes} n] == 1} {
		$w_.progress config -to $n
	    } elseif {[scan $text {read %d bytes} n] == 1} {
		$w_.progress config -value $n
	    } elseif {[scan $text {url: %s} url] == 1} {
	    	catch {
		    set fd [open $logfile_name_ a+]
		    puts $fd "[clock format [clock seconds]]\n$itk_option(-catalog)\n$url\n\n"
		    ::close $fd
		} 
	    }
	}
	$w_.progress config -text $text
	update idletasks
    }


    # This method could be used by a plugin to set the contents of the listing
    # without doing a query.
    # The arguments are the table headings (list of strings) and the table data
    # (list of rows, where each row is a list of values). The "more" flag is
    # optional and indicates that there would be more data.

    public method set_info {headings info {more 0}} {
	query_done "" $headings $info $more
    }
    


    # This method is called when the background query is done.
    #
    # Args:
    #
    #   errmsg - If this is not empty, it contains an error message and
    #   the following args should be ignored. If there were no
    #   errors, this arg is an empty string.
    #
    #   headings  - are the column headings 
    #
    #   info - is a list of rows (result of query)
    #
    #   more - is a flag set to 1 if there were more rows available that were
    #   not returned due to the maxrows limit set.

    public method query_done {errmsg headings info more} {
	if {"$errmsg" != ""} {
	    # check if we need user/passwd info. errmsg should have the format:
	    # "Authorization Required for <realm> at <host>"
	    if {[regsub {Authorization Required} $errmsg {Enter username} msg]} {
		lassign [passwd_dialog $msg] username passwd
		if {"$username" != "" && "$passwd" != ""} {
		    lassign $errmsg {} {} {} realm {} host
		    $w_.cat authorize $username $passwd $realm $host
		    $searchopts_ search
		}
	    } else {
		set_state normal
		catch {$results_ config -title "Search Results"}
		# error messages starting with "***" are only displayed in progress win
		if {"[string range $errmsg 0 2]" != "***"} {
		    error_dialog $errmsg $w_
		}
		after 0 [list $w_.progress config -text $errmsg]
	    }
	} else {
	    if {! $iscat_} {
		# for image servers, the info is the image file name
		set filename $info
		# load the image and remove the temp file
		display_image_file $filename
		catch {file delete $filename}
	    } else {
		busy {
		    set prev_headings $headings_
		    set headings_ $headings
		    set info_ $info
		    
		    # update table
		    $results_ config -headings $headings_
		    if {$reset_columns_} {
			set reset_columns_ 0
			reset_table
		    }

		    if {"$prev_headings" != "$headings_"} {
			$results_ update_options
			set_menu_states
		    }

		    $results_ config -info $info
		    
		    # need to know the equinox of the results, if using world coords
		    if {[$w_.cat iswcs]} {
			lassign [$w_.searchopts get_pos_radius] {} {} equinox
			$results_ config -equinox $equinox
		    } 

		    if {$more} {
			set more "+"
		    } else {
			set more ""
		    }
		    $results_ config -title "Search Results ([$results_ total_rows]$more)"

		    # note column indexes in array (use upper case to simplify search)
		    catch {unset col_}
		    set n -1
		    foreach i $headings_ {
			set col_($i) [incr n]
		    }
		    
		    # plot stars
		    #puts "plot time: [expr [lindex [time plot] 0]/1000000.]"
		    plot
		}
	    }

	    set_state normal
	}
    }

    
    ########################## Modified for vmmps #######################
    # start the catalog search based on the current search options
    # and display the results in the table.

    public method search {args} {


        global catname convert
        
	#puts "Inside Search" 

        # start the query in the background
	catch {
	    $results_ config -title "Searching..."
	    $results_ clear
	}
	set_state disabled
	set servtype [$w_.cat servtype]
	
	if {"$servtype" == "local"} {
	    $w_.progress config -text "Searching catalog..."
	} elseif {"$servtype" == "imagesvr"} {
	    $w_.progress config -text "Attempting to contact image server..."
	} else {
	    $w_.progress config -text "Attempting to contact catalog server..."
	}

	# AAG - Temporal
        set catname $itk_option(-catalog)
	#puts "DB> Searching in catalog: $catname"

        if {$convert == "yes"} {
            $this conv $catname
            set convert no
        }

        $w_.progress look_busy
        $searchopts_ search
    }
 
    # Get the requested image from the image server based on the given arguments
    # for the world coord position or name and the given width and height

    public method getimage_from_args {ra dec name equinox width height} {
	$searchopts_ set_pos_width_height [list $ra $dec $equinox $width $height $name]
	search
    }


    # If the given name is the name of a local catalog, check that the file exists,
    # and if not, ask the user for a new path name or remove it from the menu.
    
    public proc check_local_catalog {name {id ""} {classname vmAstroCat} {debug 0} 
				     {tcs_flag 0} {type "catalog"} {w ""}} {
	if {"[$astrocat_ servtype $name]" != "local"} {
	    return 0
	}
	set file [$astrocat_ url $name]
	if {[file isfile $file]} {
	    return 0
	}

	set msg "The catalog file [file tail $file] does not exists. \
                     Do you want to specify a new name?"
	switch [choice_dialog $msg {{Specify a new name} Cancel {Remove from menu}}] {
	    {Specify a new name} {
		set file [filename_dialog]
		if {[file isfile $file]} {
		    $astrocat_ entry remove $name
		    new_catalog $file $id $classname $debug $tcs_flag $type $w
		    if {[winfo exists .catinf]} {
			.catinf reinit_tree
		    }
		}
	    }
	    Cancel {
		return 1
	    }
	    {Remove from menu} {
		$astrocat_ entry remove $name
		if {[winfo exists .catinf]} {
		    .catinf reinit_tree
		}
	    }
	}

	# update config file and menus
	cat::CatalogInfo::save "" $w 0
	update_catalog_menus
	return 1
    }

    
    # set the values for the position and radius entries from the given
    # list, which should be in the format {ra dec equinox radius} if
    # we are using wcs, otherwise {x y radius}.

    public method set_pos_radius {list} {
	$searchopts_ set_pos_radius $list
    }


    # set the values for the position, width and height entries from 
    # the given list, which should be in the format 
    # {ra dec equinox width height (in arcmin)} for wcs, or {x y width height},
    # for pixel coordinates. 

    public method set_pos_width_height {list} {
	$searchopts_ set_pos_width_height $list
    }


    # This method is called whenever a result row is selected.
    # Note the selected values and enable/disable some buttons

    public method select_result_row {} {
	$w_.results select_result_row

	if {! [info exists col_(PREVIEW)] && ! [info exists col_(MORE)]} {
	    return
	}
	set row [lindex [$results_ get_selected] 0]
	if {[llength $row] == 0} {
	    return
	}
	set preview [get_col PREVIEW $row]
	set more [get_col MORE $row]
	set object_name_ [lindex $row 0]

	# note: HST returns P=http:....
	if {! [regsub {P=(.*)} $preview {\1} preview_url_]} {
	    set preview_url_ $preview
	}
	if {"$preview_url_" == ""} {
	    $w_.preview config -state disabled
	} else {
	    $w_.preview config -state normal
	}

	# note: HST returns M=http:....
	if {! [regsub {M=(.*)} $more {\1} more_url_]} {
	    set more_url_ $more
	}
	if {"$more_url_" == ""} {
	    $w_.more config -state disabled
	} else {
	    $w_.more config -state normal
	}
    }


    # assuming there is a field called "preview" that is the URL of an
    # image, get the image or table data for the selected line over HTTP 
    # and display it
    # 
    # The preview data may be an image or other data, such as a tab table to
    # display as a graph (like the "cuts" (RtdImageSpectrum) graph).

    public method preview {} {
	if {[$results_ num_selected] == 0} {
	    warning_dialog "Please select an object from the list first" $w_
	    return
	}
	if {! [info exists col_(PREVIEW)]} {
	    warning_dialog "Previewing is not supported for this catalog" $w_
	    return
	}
	if {"$preview_url_" == ""} {
	    warning_dialog "No preview for this object" $w_
	    return
	}
	
	# if it is a file URL, we can just get the file
	if {[string first file: $preview_url_] == 0} {
	    set filename [string range $preview_url_ 5 end]
	    preview_done 0 [list $filename image/x-fits]
	} else {
	    # HTTP: do it in the background, since it may take a while
	    set_state disabled
	    $w_.progress config -text "Attempting to contact server..."
	    $w_.progress look_busy

	    # set up the feedback pipe
	    set_feedback on

	    $w_.batch bg_eval [code $this get_preview]
	}
    }

    
    # get the preview image given by $preview_url_ into a temp file

    protected method get_preview {} {
	# get the preview and call preview_done
	$w_.cat getpreview -url $preview_url_
    }
    

    # This method is called when we have received the preview data
    # The "status" argument is the status of the background http get operation (0 if ok).
    # The result is a list of {filename Content-type}, where filename contains the data
    # and Content-type indicates the type of the preview data. Decompression is already
    # taken care of, so we only expect to get a FITS image here or a tab table to plot.
    #
    # If we get an authorization error, we ask the user to type in a username and
    # password and retry the operation with the new info. This is needed for some
    # HTTP server sites that have special restrictions on data access (especially
    # image access).

    protected method preview_done {status result} {
	set_feedback off

	if {$status} {
	    # check if we need user/passwd info. errmsg should have the format:
	    # "Authorization Required for <realm> at <host>"
	    if {[regsub {Authorization Required} $result {Enter username} msg]} {
		lassign [passwd_dialog $msg] username passwd
		if {"$username" != "" && "$passwd" != ""} {
		    lassign $result {} {} {} realm {} host
		    $w_.cat authorize $username $passwd $realm $host
		    preview
		}
	    } else {
		error_dialog $result $w_
	    }
	} else {
	    if {[llength $result] != 2} {
		error_dialog "error getting preview data (result = $result)" $w_
	    } else {
		lassign $result filename type
		if {"$type" == "image/x-fits"} {
		    # load the image and remove the temp file
		    display_image_file $filename
		} elseif {"$type" == "text/tab-separated-values" \
			      || "$type" == "text/x-starbase" \
			      || "$type" == "text/plain" \
			      || "$type" == ""} {
		    PreviewPlot $w_.pplot[incr count_] \
			-file $filename \
			-name $object_name_ \
			-shorthelpwin $this \
			-transient 1
		} else {
		    error_dialog "unrecognized Content-type for preview data: $type" $w_
		}
		if {[string first file: $preview_url_] != 0} {
		    # if the image was retrieved via http, remove it
		    catch {file delete $filename}
		}
	    }
	}
 	set_state normal
	
	# restore preview button state...
	select_result_row
    }

    
    # send a URL to be displayed by netscape

    public proc send_to_netscape {url} {
	if {[catch {exec netscape -remote "openURL($url)"} msg1]} {
	    if {! [string match $msg1 "netscape: warning"]} {
		if {[catch {exec netscape $url &} msg2]} {
		    warning_dialog "couldn't start netscape: $msg1"
		} else {
		    info_dialog "Starting netscape..."
		}
	    }
	} else {
	    info_dialog "Sending help URL to netscape..."
	}
    }
    
    
    # use the "more" URL to display more info about an object using netscape
    # or mosaic

    public method more {} {
	if {[$results_ num_selected] == 0} {
	    warning_dialog "Please select an object from the list first" $w_
	    return
	}
	if {"$more_url_" == ""} {
	    warning_dialog "No more information is available for this object" $w_
	    return
	}
	# start netscape
	$w_.progress config -text "sending URL to netscape..."
	update idletasks
	send_to_netscape $more_url_
    }


    
    # return the value for the given column name in the given row

    public method get_col {name row} {
	if {[info exists col_($name)]} {
	    return [lindex $row $col_($name)]
	} 
    }


    # This member proc is used to open a window for the named catalog, 
    # or reuse the existing one for the catalog, if it is already open.
    #
    # name is the long name of catalog from config file
    #
    # id is an optional unique id to be associated with a new catalog widget.
    #
    # classname is the name of the vmAstroCat subclass to use to create new
    # catalog widgets (defaults to "vmAstroCat").
    #
    # debug is a flag: if true, run queries in foreground
    #
    # tcs_flag may be set to true if the catalog is TCS format
    #
    # type is the catalog serv_type (here it is enough to specify "catalog" or
    # "imagesvr".
    #
    # w should be the top level window of the caller, if specified.

    public proc new_catalog {name {id ""} {classname vmAstroCat} {debug 0} {tcs_flag 0} 
		      {type "catalog"} {w ""}} {


	# if it is a local catalog, make sure it exists still, or get a new name
	if {[check_local_catalog $name $id $classname $debug $tcs_flag $type $w] != 0} {
	    return
	}

	set i "$name,$id"
	if {[info exists instances_($i)] && [winfo exists $instances_($i)]} {
	    utilRaiseWindow $instances_($i)
	    if {"[$instances_($i).cat servtype]" == "local"} {
		# for local catalogs, search automatically when opened
		$instances_($i) search
	    }

	    return
	}

	if {[catch {$astrocat_ open $name} msg]} {
	    error_dialog $msg .skycat1
	    return
	}

	# If $w was specified, put the window under that top level window
	# so we get the window numbers right (for cloning, see TopLevelWidget).
	if {[winfo exists $w]} {
	    set instname $w.vmac[incr n_instances_]
	} else {
	    set instname .vmac[incr n_instances_]
	}
	#puts "DB> Window: $instname"
	set instances_($i) \
	    [$classname $instname \
		 -id $id \
		 -debug $debug \
		 -catalog $name \
		 -catalogtype $type \
		 -tcs $tcs_flag \
		 -transient 0 \
		 -center 0]

    }
    
    
    # Return a Tcl list of instances of this class. By default
    # (and for backward compatibility) only catalog windows are
    # included - not image servers. If "choice" is "all", then
    # all instances are returned. If choice is imagesvr, only 
    # image server windows are returned.

    public proc instances {{choice catalogs}} {
	set list {}
	if {[info exists instances_]} {
	    foreach i [array names instances_] {
		if {[winfo exists $instances_($i)]} {
		    if {[$instances_($i) iscat]} {
			# instance of a catalog window
			if {"$choice" == "catalogs" || "$choice" == "all"} {
			    lappend list $instances_($i)
			}
		    } else {
			# instance of an image server window
			if {"$choice" == "imagesvr"} {
			    lappend list $instances_($i) 
			}
		    }
		}
	    }
	}
	return $list
    }


    # Return the widget instance for the given catalog, if found,
    # otherwise an empty string

    public proc get_instance {catalog} {
	foreach w [cat::vmAstroCat::instances all] {
	    if {"[$w cget -catalog]" == "$catalog"} {
		return $w
	    }
	}
    }

    
    # Return 1 if this window is for a searchable catalog, 
    # and 0 otherwise (in which case it must be an image server window).
    
    public method iscat {} {
	return $iscat_
    }


    # ---------------------------------------------------------------
    # The following methods deal with images and plotting of objects
    # in an image and are meant to be defined in a derived class.
    # See the skycat/skycat/interp/library/SkySearch.tcl for an example.
    # ---------------------------------------------------------------

    
    # plot the stars/objects found in the previous search in a (image) window.
    # The symbols to use are taken from the config file.
    # (To be defined in a subclass)

    public method plot {} {
    }
    

    # display the given image file (To be defined in a subclass)

    public method display_image_file {filename} {
    }


    #  return the equinox

    public method get_equinox {} {
	return [component searchopts get_equinox]
    }


    #  return the name of the TableList

    public method get_table {} {
	if {[info exists itk_component(results)]} {
	    return $itk_component(results)
	}
	error "This catalog is an image server"
    }


    # -- options --

    # name of catalog
    itk_option define -catalog catalog Catalog "" {
	# make sure we use full path name for local catalogs
	set f $itk_option(-catalog)
	if {[file exists $f] && "[string index $f 0]" != "/"} {
	    set itk_option(-catalog) [pwd]/$f
	}
    }

    # type of catalog (catalog or archive)
    itk_option define -catalogtype catalogType CatalogType "catalog"

    # Optional unique id, used in searching for already existing catalog widgets.
    itk_option define -id id Id ""

    # list of catalog column headings (from results of most recent query)
    protected variable headings_ {}

    # result from most recent query (list of rows)
    protected variable info_ {}

    # font to use for labels
    itk_option define -labelfont labelFont LabelFont -Adobe-helvetica-bold-r-normal--12*

    # font to use for values
    itk_option define -valuefont valueFont ValueFont -Adobe-helvetica-medium-r-normal--12*

    # font to use for ra,dec labels
    itk_option define -wcsfont wcsFont WcsFont -*-symbol-*-*-*-*-14-*-*-*-*-*-*-*

    # set the width for  displaying labels
    itk_option define -labelwidth labelWidth LabelWidth 12

    # set the anchor for labels
    itk_option define -anchor anchor Anchor e

    # flag: if true, run queries in foreground for better debugging
    itk_option define -debug debug Debug 0

    # if -tcs is 1, use fixed format TCS catalog listings
    itk_option define -tcs tcs Tcs {0}

    # vmAstroQuery widget used to manage search options
    protected variable searchopts_ {}

    # QueryResult widget used to display search results
    protected variable results_ {}

    # array(uppercase col name) of col index from catalog headings
    protected variable col_

    # preview URLs for selected object
    protected variable preview_url_ {}

    # more-info URLs for selected object
    protected variable more_url_ {}

    # pipe to read feedback during HTTP transfer
    protected variable rfd_

    # pipe to write feedback during HTTP transfer
    protected variable wfd_

    # current state: normal, disabled (i.e.: waiting)
    protected variable state_ {normal}

    # log file handle (used to log URLs)
    protected variable logfile_name_ 

    # flag: set at end of constructor
    protected variable initialized_ 0
    
    # index of this object in the instances_ array
    protected variable instance_idx_ {}

    # name of File menu widget
    protected variable file_menu_

    # name of Edit menu widget
    protected variable edit_menu_

    # name of Options menu widget
    protected variable options_menu_

    # name of Name Server menu widget
    protected variable ns_menu_

    # flag: set when the column headings are changed between TCS and normal
    protected variable reset_columns_ {0}

    # currently selected object (Id field in row)
    protected variable object_name_ {}

    # count used for filename generation
    protected variable count_ 0

    # flag: true if catalog is not an image server
    protected variable iscat_ 1

    # flag: true if searching is allowed
    protected variable search_state_ normal


    # -- common variables (common to all instances of this class) --
    
    # C++ astrocat object used by static member procs
    protected common astrocat_ [astrocat ::cat::.vmastrocat]
    
    # array mapping catalog name to widget/class name
    protected common instances_

    # instance count
    protected common n_instances_ 0

    # current instance name
    protected common current_instance_ {}

    # array(TopLevelWidget instance) of command used to update the
    # Data-Servers menu. This is used to make it posible to update
    # all instances of this menu in various top level windows.
    protected common catalog_menu_info_

    # flag: set to 1 after we checked for a proxy server
    protected common checked_proxies_ 0

}
